package nova.traffic.media.manage;

import nova.traffic.media.ItemMediaBuilder;
import nova.traffic.media.PlayBuilder;
import nova.traffic.media.help.*;
import nova.traffic.utils.LogUtil;
import nova.traffic.utils.NovaTrafficImp;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 媒体管理类
 */
public class PlayManager {

    private PlayBuilder playMediaBuild;  //节目build对象
    private List<String> mediaListSource; //需要上传的文件集合

    private NovaTrafficImp novaTraffic; //nova无线连接协议
    /**
     * 存在同名媒体，是否替换，默认存在不上传
     */
    private boolean isReplaceMedia = false;


    public PlayManager(NovaTrafficImp novaTraffic) {
        this.novaTraffic = novaTraffic;
        mediaListSource = new ArrayList<>();
    }

    /**
     * @param builder     自定义节目内容
     * @param novaTraffic 连接协议【客户端或者服务器】
     */
    public PlayManager(PlayBuilder builder, NovaTrafficImp novaTraffic) {
        this.playMediaBuild = builder;
        this.novaTraffic = novaTraffic;
        mediaListSource = new ArrayList<>();
    }

    /**
     * 有同名媒体是否替换，默认false
     *
     * @param replaceMedia
     * @return
     */
    public PlayManager isReplaceMedia(boolean replaceMedia) {
        isReplaceMedia = replaceMedia;
        return this;
    }

    /**
     * 设置节目build
     *
     * @param playMediaBuild 节目build
     */
    public void setPlayBuild(PlayBuilder playMediaBuild) {
        this.playMediaBuild = playMediaBuild;
    }

    /**
     * 创建一个协议策略（标准协议）
     *
     * @return 返回标准协议类型
     */
    public OnMediaStrategyImp getMediaAllStrategy() {
        return new MediaAllStrategyImpl();
    }

    /**
     * 创建一个协议策略（局部更新协议）
     *
     * @return 返回局部更新协议类型
     */
    public OnMediaStrategyImp getMediaPartStrategy() {
        return new MediaPartStrategyImpl();
    }


    /**
     * 创建一个协议策略（实时更新）
     *
     * @return 返回实时更新协议类型
     */
    public OnMediaStrategyImp getMediaRealStrategy() {
        return new MediaLiveStrategyImpl();
    }


    /**
     * @param imp 设置协议策略  MediaAllStrategyImpl 标准协议
     *            MediaPartStrategyImpl 局部更新协议
     * @return PlayManager
     */
    public PlayManager setUpdateStrategy(OnMediaStrategyImp imp) {
        playMediaBuild.mediaStrategyImp(imp);
        return this;
    }

    /**
     * 设置playMedia id
     *
     * @param playId 设置节目id
     * @return PlayManager
     */
    public PlayManager setPlayMediaId(int playId) {
        playMediaBuild.Id(playId);
        return this;
    }

    /**
     * 添加一个节目（通过build构造）
     *
     * @param builder item
     * @return PlayManager
     */
    public PlayManager addBuilderItem(ItemMediaBuilder builder) {
        playMediaBuild.addBuilderItem(builder);
        return this;
    }

    /**
     * 添加一个节目（自己构造指令，该模式下添加的指令需要媒体文件保证媒体文件已存在）
     *
     * @param pushProtocol 协议
     * @return PlayManager
     */
    public PlayManager addProtocolBuilder(String pushProtocol) {
        playMediaBuild.addProtocolBuilder(pushProtocol);
        return this;
    }


    /**
     * 获取节目播放协议
     *
     * @param strategyImp 节目策略  普通节目、局部更新，item实时更新
     * @return 返回协议
     */
    public String getProtocol(OnMediaStrategyImp strategyImp) {
        return playMediaBuild.mediaStrategyImp(strategyImp).builder().createProtocol();
    }

    /**
     * 获取节目id
     *
     * @return 节目id
     */
    public int getPlayId() {
        return playMediaBuild.builder().getId();
    }

    /**
     * 待上传的文件列表
     *
     * @return 待上传文件
     */
    public List<String> getMediaFileList() {
        /**
         * 添加参数中找到的源文件
         */
        mediaListSource.addAll(playMediaBuild.builder().getMediaFileList());
        List<String> list = mediaListSource.stream().distinct().collect(Collectors.toList());
        return list;
    }

    /**
     * 添加上传文件
     *
     * @return PlayManager
     */
    public PlayManager addMediaSource(String pathSource) {
        mediaListSource.add(pathSource);
        return this;

    }

    /**
     * 添加上传文件集合
     *
     * @return PlayManager
     */
    public PlayManager addMediaSource(List<String> pathSources) {
        mediaListSource.addAll(pathSources);
        return this;

    }

    /**
     * 标准节目，通过结构化参数设置
     * 1.上传所有附件（直接发送指令不涉及）
     * 2.发送指令
     * 备注：addProtocolBuilder 直接添加指令，无法设置更新策略，以实际指令为准
     *
     * @return 返回播放结果
     */
    public int play() {
        String protocol = playMediaBuild.mediaStrategyImp(getMediaAllStrategy()).builder().createProtocol(); //发送的指令
        LogUtil.log("协议内容："+protocol);
        int playId = getPlayId();  //节目id

        StringBuffer sendFileLog = new StringBuffer();

        uploadMediaFile(sendFileLog);


        LogUtil.log("发送文件结果" + sendFileLog.toString());
        LogUtil.log("普通节目播放内容：\n" + protocol);
        int i = novaTraffic.sendPlayList(playId, protocol);
        LogUtil.log("播放结果" + i);
        return i;

    }

    /**
     * 局部更新，通过结构化参数设置
     * 1.上传所有附件（直接发送指令不涉及）
     * 2.发送指令
     *
     * @return 返回播放结果
     */
    public int regionUpdate() {
        String protocol = playMediaBuild.mediaStrategyImp(getMediaPartStrategy()).builder().createProtocol(); //发送的指令
        return update(protocol);

    }

    /**
     * 实时更新
     *
     * @return 返回播放结果
     */
    public int liveUpdating() {
        String protocol = playMediaBuild.mediaStrategyImp(getMediaRealStrategy()).builder().createProtocol(); //发送的指令
        return update(protocol);

    }

    /**
     * 更新节目
     *
     * @param protocol
     * @return 返回播放结果
     */
    private int update(String protocol) {
        LogUtil.log("协议内容："+protocol);
        int playId = getPlayId();  //节目id
        StringBuffer sendFileLog = new StringBuffer();


        uploadMediaFile(sendFileLog);


//        LogUtil.log("局部更新发送文件结果" + sendFileLog.toString());
//        LogUtil.log("局部更新播放内容：\n" + protocol);
//        LogUtil.log(sendFileLog.toString());
//        LogUtil.log(protocol);
        int i = novaTraffic.sendLocalUpdate(playId, protocol);
        LogUtil.log("局部更新播放结果：" + i);
        return i;
    }


    /**
     * 上传媒体文件
     * @param sendFileLog
     */
    private void uploadMediaFile(StringBuffer sendFileLog) {
        List<String> mediaFileList = getMediaFileList(); //待上传的附件
        if (isReplaceMedia) {
            mediaFileList.forEach(sourcePath -> {
                File file = new File(sourcePath);
                int res = novaTraffic.sendFile(file, file.getName());
                sendFileLog.append(res).append("\n");
            });
        } else {
            List<String> allMediaFileName = novaTraffic.getAllMediaFileName();
            mediaFileList.forEach(sourcePath -> {
                File file = new File(sourcePath);
                if (allMediaFileName.contains(file.getName())) {
                    return;
                }
                int res = novaTraffic.sendFile(file, file.getName());
                sendFileLog.append(res).append("\n");
            });
        }
    }


}
