package nova.traffic.server;

import nova.traffic.utils.LogConfig;
import nova.traffic.utils.LogUtil;

import java.io.IOException;
import java.net.InetAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectableChannel;
import java.nio.channels.SocketChannel;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;

public class Channel {
    private static final int DEFAULT_READ_BUFFER = 1024;

    protected Map<String, Queue<ByteBuffer>> msgMap = new HashMap<String, Queue<ByteBuffer>>();
    protected Map<String, MessageParser> parserMap = new HashMap<String, MessageParser>();

    protected int onWritable(SelectableChannel channel) {
        try {
            if (msgMap == null || msgMap.isEmpty()) {
                return 0;
            }
            SocketChannel socketChannel = (SocketChannel) channel;
            InetAddress address = socketChannel.socket().getInetAddress();
            String ip = address.getHostAddress();
            int port = socketChannel.socket().getPort();
            synchronized (msgMap) {
                Queue<ByteBuffer> queue = msgMap.get(deviceKey(ip, port));
                if (queue == null) {
                    return 0;
                }
                ByteBuffer bf = queue.peek();
                if (bf == null || !bf.hasRemaining()) {
                    return 0;
                }
                int position = bf.position();
                int count = socketChannel.write(bf);
                if (LogConfig.debug) {
                    byte[] buffer = new byte[count];
                    bf.position(position);
                    bf.get(buffer, 0, count);
                    LogUtil.log("写数据到 " + ip + ":" + port + "：" + LogUtil.byteToStr(buffer, count));
                }
                if (!bf.hasRemaining()) {
                    queue.poll();
                } else {
                    LogUtil.log("数据未写完毕，等待下一次再写");
                }
                return count;
            }
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
            LogUtil.log("写数据异常" + e.toString());
        }
        return 0;
    }

    protected void onReadable(SelectableChannel channel) {
        SocketChannel socketChannel = (SocketChannel) channel;
        InetAddress address = socketChannel.socket().getInetAddress();
        String ip = address.getHostAddress();
        int port = socketChannel.socket().getPort();
        synchronized (parserMap) {
            int size = DEFAULT_READ_BUFFER;
            MessageParser socketMessage = parserMap.get(deviceKey(ip, port));

            int count = DEFAULT_READ_BUFFER;
            while (count == size) {
                ByteBuffer buffer = ByteBuffer.allocate(size);
                try {
                    count = socketChannel.read(buffer);
//					LogUtil.log("读取到数据："+count);
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    LogUtil.log("远程主机强迫关闭了一个现有的连接" + e.toString());
//					e.printStackTrace();
                    try {
                        channel.close();
                        onReadError(ip, port);
                    } catch (IOException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                    break;
                }
                if (count < 0) {
                    try {
                        channel.close();
                        LogUtil.log(ip + ":" + port + "客户端已关闭");
                        onReadError(ip, port);
                    } catch (IOException e1) {
                        // TODO Auto-generated catch block
                        e1.printStackTrace();
                    }
                }
                if (count <= 0) {
                    break;
                }
                buffer.rewind();
                buffer.limit(count);

                if (socketMessage != null) {
                    socketMessage.put(buffer, count);
                    LogUtil.log("收到 " + ip + ":" + port + " 的消息：" + LogUtil.byteToStr(buffer.array(), count));
                    socketMessage.parse();
                } else {
                    LogUtil.log("丢弃客户端 " + ip + ":" + port + "的消息：" + LogUtil.byteToStr(buffer.array(), count));
                }
            }
        }
    }

    synchronized void send(String ip, int port, byte[] bytes) {
        if (LogConfig.debug)
            LogUtil.log("发送： " + ip + ":" + port + " 的消息：" + LogUtil.byteToStr(bytes));
//            LogUtil.log("发送： " + ip + ":" + port + " 的消息" );
        synchronized (msgMap) {
            Queue<ByteBuffer> queue = msgMap.get(deviceKey(ip, port));
            if (queue == null) {
                queue = new LinkedList<ByteBuffer>();
                msgMap.put(deviceKey(ip, port), queue);
            }
            ByteBuffer buffer = ByteBuffer.allocate(bytes.length);
            buffer.put(bytes);
            buffer.position(0);
            queue.offer(buffer);
        }
    }

    void putMessageParser(String ip, int port, MessageParser socketMessage) {
        synchronized (parserMap) {
            parserMap.put(deviceKey(ip, port), socketMessage);
        }
    }


    void remove(String ip, int port) {
        synchronized (msgMap) {
            msgMap.remove(deviceKey(ip, port));
        }
        synchronized (parserMap) {
            parserMap.remove(deviceKey(ip, port));
        }
    }

//	/**
//	 * 初始化解析器
//	 * @param ip
//	 */
//	protected void initMessageParser(String ip) {
//		synchronized (parserMap) {
//			MessageParser parser = parserMap.get(ip);
//			if(parser != null) {
//				parser.init();
//			}
//		}
//	}

    /**
     * 读取错误
     * @param ip ip
     * @param port port
     */
    protected void onReadError(String ip, int port) {
        if (ip == null) {
            return;
        }
        synchronized (parserMap) {
            MessageParser parser = parserMap.get(deviceKey(ip, port));
            if (parser != null) {
                parser.onReadError();
            }
        }
    }

    protected void notifyConnect(String ip, int port) {
        synchronized (parserMap) {
            MessageParser parser = parserMap.get(deviceKey(ip, port));
            if (parser != null) {
                parser.notifyConnect();
            }
        }
    }

    /**
     * 设备的key
     *
     * @param ip ip
     * @param port port
     * @return 键值对
     */
    protected String deviceKey(String ip, int port) {
        return ip + ":" + port;
    }
}
