package nova.traffic.server;

import nova.traffic.utils.AesUtil;
import nova.traffic.utils.DesUtil;
import nova.traffic.utils.LogUtil;
import nova.traffic.utils.NovaUtils;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;


public class MessageParser {
	//消息缓冲区大小
	private static final int DEFLAUT_PARK_BLOCK_SIZE = 65535 * 3;


	private static final int PACKET_MIN_SIZE = 7;
	private static final int PACKET_WITHOUT_WHAT_AND_DATA = 6;
	private static final int PACKET_WHAT_OFFSET = 3;
	private static final int PACKET_DATA_OFFSET = 4;
	private static final byte DES_ADD_SIZE = 7;
	private static final int PACKET_WITHOUT_DATA_SIZE = 7;

	private static final byte PACKET_START_FLAG = (byte) 0xAA;
	private static final byte PACKET_END_FLAG = (byte) 0xCC;
	private static final byte PACKET_TRANSFER_FLAG = (byte) 0xEE;
	private static final byte WHAT_SECRET_DES_MD5_REQ = 0x71;

	private static final int PACKET_CRC_SIZE = 2;

	private volatile String mEncodeSecret = null;

	private int encodeType = 0;

	private ByteBuffer buffer;

	private MessageParserCallback callback;

	interface MessageParserCallback{
		void onReceive(byte[] bytes);
		void onReadMessageError();
		void onConnect();
	}

	public MessageParser() {
		buffer = ByteBuffer.allocate(DEFLAUT_PARK_BLOCK_SIZE);
	}

	public void init() {
		buffer = ByteBuffer.allocate(DEFLAUT_PARK_BLOCK_SIZE);
	}

	public void setEncodeSecret(int encodeType, String encodeSecret) {
		if(this.mEncodeSecret != null) {
			if(!this.mEncodeSecret.equals(encodeSecret)) {
				this.mEncodeSecret = encodeSecret;
				init();
			}
		}else {
			this.mEncodeSecret = encodeSecret;
			init();
		}

	}

	void setCallback(MessageParserCallback callback) {
		this.callback = callback;
	}

	void put(ByteBuffer buffer2, int count) {
		if(buffer.remaining() < count) {
			LogUtil.log("数据接收区域已满");
			return;
		}
		buffer.put(buffer2);
	}
	/**
	 * 解析数据
	 */
	void parse() {
		int size = buffer.position();
		if(size <= 0) {
			LogUtil.log("没有需要解析的数据");
			return;
		}
		byte[] bytes = new byte[size];
		buffer.position(0);
		buffer.get(bytes);
		List<Node> list = getNodeList(bytes);
		compact(size,list);
		postMessage(bytes,list);
	}
	/***
	 * 将分割出来的消息抛出去
	 * @param bytes
	 * @param list
	 */
	private void postMessage(byte[] bytes, List<Node> list) {
		for(Node node:list) {
			int count = node.endIndex - node.startIndex + 1;
			byte[] msg = new byte[count];
			System.arraycopy(bytes, node.startIndex, msg, 0, count);
			byte[] recvByte = recvPackage(msg);
			if(recvByte != null) {
				notify(recvByte);
			}else {
				LogUtil.log("没有解析出数据");
			}
		}
	}

	private void notify(byte[] recvByte) {
		if(null != callback) {
			callback.onReceive(recvByte);
		}
	}

	/**
	 * 处理剩余消息，干掉已处理的消息
	 * @param size
	 * @param list
	 */
	private void compact(int size, List<Node> list) {
		int lenght  = 0;
		if(!list.isEmpty()) {
			Node lastNode = list.get(list.size()-1);
			lenght = size - (lastNode.endIndex + 1);
			buffer.position(lastNode.endIndex + 1);
			buffer.compact();
		}else {
			lenght = size;
		}
		buffer.position(lenght);
	}

	/**
	 * 获取协议起止节点列表
	 * @param bytes
	 * @return
	 */
	private List<Node> getNodeList(byte[] bytes) {
		int index = -1;
		List<Node> list = new ArrayList<Node>();
		Node node = null;
		int remain = bytes.length;
		while ( ++index < remain) {
			byte b = bytes[index];
			if(PACKET_START_FLAG == b) {
				node = new Node();
				node.startIndex = index;
			}else if(PACKET_END_FLAG == b) {
				//结束符
				int end = index + PACKET_CRC_SIZE;
				if(end < remain && node != null) {
					node.endIndex = end;
					index = end;//跳过两位crc校验
					list.add(node);
				}
			}
		}
		return list;
	}

	private byte[] recvPackage(byte[] buffer) {
		int count = buffer.length;
		int tempSize = 0;
		int copySize = 0;

		// 起始符，结束符，长度校验
		if (count < PACKET_MIN_SIZE) {
			LogUtil.log("收到的数据长度小于通信数据包最小长度！");
			return null;
		}

		// CRC校验
		short mCrc = NovaUtils.pubCalcCRC(buffer, count - PACKET_CRC_SIZE);
		short mCrcCode = NovaUtils.bytesToShort(buffer, count - PACKET_CRC_SIZE);
		if (mCrc != mCrcCode) {
			LogUtil.log("CRC校验失败！");
			return null;
		}
		// decode 0xee
		byte[] mRecvBuffer = new byte[count];
		while (tempSize < count - 2) {
			if (buffer[tempSize] != PACKET_TRANSFER_FLAG) {
				mRecvBuffer[copySize] = buffer[tempSize];

			} else {
				tempSize++;
				mRecvBuffer[copySize] = buffer[tempSize];
				if (mRecvBuffer[copySize] == 0x0a) {
					mRecvBuffer[copySize] = (byte) 0xAA;
				} else if (mRecvBuffer[copySize] == 0x0c) {
					mRecvBuffer[copySize] = (byte) 0xcc;
				} else if ((mRecvBuffer[copySize] == 0x0e)) {
					mRecvBuffer[copySize] = (byte) 0xee;
				} else {
					// LOGGER.warn("数据转义0xEE错误！");
					return null;
				}
			}
			tempSize++;
			copySize++;
		}

		// crc 2位
		mRecvBuffer[copySize++] = buffer[tempSize++];
		mRecvBuffer[copySize++] = buffer[tempSize++];

		// decode DES 设置加密属性不加密
		if (encodeType == 1 && mEncodeSecret != null && copySize > PACKET_MIN_SIZE && buffer[3] != WHAT_SECRET_DES_MD5_REQ) {
			try {
				byte[] tempOutData = DesUtil.decrypt(mRecvBuffer, PACKET_DATA_OFFSET, copySize - PACKET_MIN_SIZE,
						mEncodeSecret.getBytes());
				System.arraycopy(tempOutData, 0, mRecvBuffer, PACKET_DATA_OFFSET, tempOutData.length);
				copySize = tempOutData.length + PACKET_WITHOUT_DATA_SIZE;

			} catch (Exception e) {
				return null;
			}
		}
		else if (encodeType == 2 && mEncodeSecret != null && copySize > PACKET_MIN_SIZE && buffer[3] != WHAT_SECRET_DES_MD5_REQ){
			try {
				byte[] tempOutData = AesUtil.decrypt(mRecvBuffer, PACKET_DATA_OFFSET, copySize - PACKET_MIN_SIZE,
						mEncodeSecret.getBytes());
				System.arraycopy(tempOutData, 0, mRecvBuffer, PACKET_DATA_OFFSET, tempOutData.length);
				copySize = tempOutData.length + PACKET_WITHOUT_DATA_SIZE;

			} catch (Exception e) {
				return null;
			}
		}

		// 外传结构
		int outSize = copySize - PACKET_WITHOUT_WHAT_AND_DATA;
		byte[] mBuffer = new byte[outSize];
		System.arraycopy(mRecvBuffer, PACKET_WHAT_OFFSET, mBuffer, 0, outSize);
		return mBuffer;
	}

	public void onReadError() {
		if(callback != null) {
			callback.onReadMessageError();
		}
	}

	public void notifyConnect() {
		if(callback != null) {
			callback.onConnect();
		}
	}

	private static class Node{
		int startIndex,endIndex;
	}


}
