package nova.traffic.server;

import nova.traffic.utils.LogUtil;
import nova.traffic.utils.NovaTrafficServer;

import java.io.IOException;
import java.nio.channels.SocketChannel;

public class NovaDevice extends MessageChannel {
    /**
     * 心跳超时时间
     */
    private final static int HEAT_TIMEOUT = 60 * 1000;
    private ConnectListener connectListener;
    /**
     * 是否连接上
     */
    private volatile boolean isConnected = false;

    private int timeOut = 10 * 1000;

    private volatile int what = -1;

    private byte[] receiveBuf;

    private Object lock = new Object();
    /**
     * 异步接收锁
     */
    private Object asynLock = new Object();
    /**
     * 异步接收回调
     */
    private Callback asynCallback;

    private AllCallback asynAllCallback;

    /**
     * 是否异步接收，默认为否，仅在异步接收数据时时true
     */
    private volatile boolean isAsynReceive = false;
    /**
     * 异步接收所有消息
     */
    private volatile boolean isAsynReceiveAll = false;
    /**
     * 通道
     */
    private SocketChannel socketChannel;

    private NovaTrafficServer trafficServer;
    /**
     * 设备名称
     */
    private String deviceName;
    /**
     * 设备接入时间，创建设备的时候记录
     */
    private long accessTime;

    NovaDevice(String ip, int port, SocketChannel socketChannel) {
        super(ip, port);
        this.socketChannel = socketChannel;
        accessTime = System.currentTimeMillis();
    }

    public NovaDevice(String ip, int port) {
        super(ip, port);

    }

    public synchronized NovaTrafficServer obtainTrafficServer() {
        if (trafficServer == null) {
            trafficServer = new NovaTrafficServer(this);
        }
        return trafficServer;
    }

    public SocketChannel getSocketChannel() {
        return socketChannel;
    }

    public int getTimeOut() {
        return timeOut;
    }


    public void setTimeOut(int timeOut) {
        this.timeOut = timeOut;
    }

    @Override
    public String toString() {
        return "NovaDevice{" +
                "ip=" + ip + ":port=" + port +
                ", deviceName='" + deviceName + '\'' +
                '}';
    }

    @Override
    public synchronized void send(byte[] outbuf) throws Exception {
        if (!isConnected) {
            throw new Exception("设备未连接");
        } else {
            super.send(outbuf);
        }
    }

    /**
     * 同步接收
     *
     * @param what what
     * @return 心跳
     */
    public synchronized byte[] receive(short what) {
        synchronized (lock) {
            receiveBuf = null;
            this.what = what;
            try {
                lock.wait(timeOut);
            } catch (InterruptedException e) {
                LogUtil.log("receive:"+e.toString());
                // Restore interrupted state...
                Thread.currentThread().interrupt();

            }
            this.what = -1;
        }
        return receiveBuf;
    }

    /**
     * 异步接收指定消息
     *
     * @param what
     * @param asynCallback
     */
    public synchronized void receiveAsync(byte what, Callback asynCallback) {
        receiveAsync(what, timeOut, asynCallback);
    }

    /**
     * 异步接收指定消息
     *
     * @param what
     * @param timeOut
     * @param asynCallback
     */
    public synchronized void receiveAsync(byte what, int timeOut, Callback asynCallback) {
        synchronized (asynLock) {
            this.asynCallback = asynCallback;
            this.what = what;
            isAsynReceive = true;
            try {
                asynLock.wait(timeOut);
            } catch (Exception e) {
                LogUtil.log("receive:"+e.toString());
                // Restore interrupted state...
                Thread.currentThread().interrupt();
            }
            this.what = -1;
            isAsynReceive = false;
            asynCallback = null;
        }
        return;
    }

    public synchronized void receiveAllAsync(AllCallback asynAllCallback) {
        receiveAllAsync(timeOut, asynAllCallback);
    }

    /**
     * 异步接收所有消息
     *
     * @param timeOut
     * @param asynAllCallback
     */
    public synchronized void receiveAllAsync(int timeOut, AllCallback asynAllCallback) {
        synchronized (asynLock) {
            this.asynAllCallback = asynAllCallback;
            isAsynReceiveAll = true;
            try {
                asynLock.wait(timeOut);
            } catch (Exception e) {
                LogUtil.log("receiveAllAsync:" + e.toString());
                Thread.currentThread().interrupt();
            }
            this.what = -1;
            isAsynReceiveAll = false;
            this.asynAllCallback = null;
        }
        return;
    }

    /**
     * 异步接收结束，在异步接收完成时一定要调用，否则异步接口一直会等到超时
     */
    public synchronized void asyncReceiveEnd() {
        synchronized (asynLock) {
            asynLock.notifyAll();
        }
    }

    @Override
    protected void onReceiveBuffer(short what, byte[] bytes) {
        LogUtil.log("收到解析后的消息" + ip + " : " + LogUtil.byteToStr(bytes, bytes.length));
        if (isAsynReceiveAll) {
            //异步拦截所有消息
            asynReceiveAll(what, bytes);
            return;
        }
        if (this.what != -1) {
            if (this.what == what) {
                if (isAsynReceive) {
                    asynReceive(bytes);
                } else {
                    synReceive(bytes);
                }
            } else {
                LogUtil.log("丢弃不对应消息：" + ip);
            }
        } else {
            LogUtil.log("丢弃消息：" + ip + ":" + port);
        }
    }

    /**
     * 异步拦截所有消息
     *
     * @param what
     * @param bytes
     */
    private void asynReceiveAll(short what, byte[] bytes) {
        if (null != asynAllCallback) {
            boolean ret = asynAllCallback.onReceive(what, bytes);
            if (!ret) {
                //不再异步接收
                synchronized (asynLock) {
                    asynLock.notifyAll();
                }
            }
        }
    }

    /**
     * 处理异步消息
     *
     * @param bytes
     */
    private void asynReceive(byte[] bytes) {
        synchronized (asynLock) {
            if (this.what == -1) {
                LogUtil.log("异步消息超时：" + ip);
                return;
            }
            if (asynCallback != null) {
                //异步接收，将消息直接交给监听者来处理
                boolean ret = asynCallback.onReceive(bytes);
                if (!ret) {
                    //结束异步接收指定消息
                    asynLock.notifyAll();
                }
            }
        }
    }

    /**
     * 处理同步消息
     *
     * @param bytes
     */
    private void synReceive(byte[] bytes) {
        synchronized (lock) {
            if (this.what == -1) {
                LogUtil.log("消息超时：" + ip);
                return;
            }
            receiveBuf = new byte[bytes.length];
            System.arraycopy(bytes, 0, receiveBuf, 0, bytes.length);
            lock.notifyAll();
        }

    }

    @Override
    protected void onHeartBeat() {
        LogUtil.log(ip + " " + deviceName + " 收到心跳消息");
        if (null != connectListener && !isConnected) {
            LogUtil.log("客户端:" + ip + ":" + port + "连接成功，  名称:" + deviceName);
            isConnected = true;
            connectListener.onConnected(this, ip);
        }
        isConnected = true;

        if (connectListener != null) {
            connectListener.onReceivedHeartBeat(this, ip);
        }
    }

    @Override
    public void onDisconnect() {
        if (connectListener != null && isConnected) {
            this.isConnected = false;
            LogUtil.logTrace("设备" + ip + "断开");
            connectListener.onDisconnect(this, ip);
        } else {
            LogUtil.logTrace("试图调用onDisconnect方法，但是设备已经断开");
        }

    }

    @Override
    public void close() {
        // TODO Auto-generated method stub
        super.close();
        isConnected = false;
        if (socketChannel != null) {
            synchronized (socketChannel) {
                try {
                    socketChannel.close();
                } catch (IOException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        }
    }

    public boolean isConnected() {
        return isConnected;
    }

    public boolean enable() {
        if (getLastHeartbeatTime() != -1 && (System.currentTimeMillis() - getLastHeartbeatTime()) > HEAT_TIMEOUT) {
            LogUtil.log(ip + " " + deviceName + "超过一分钟无心跳，客户端已断开");
            return false;
        }
        return isConnected;
    }

    /**
     * 查询设备是否有效
     *
     * @return true 有效  false 无效
     */
    public boolean isUseDevice() {
        long curTime = System.currentTimeMillis();
        if (deviceName == null && curTime - accessTime > 2 * 60 * 1000) {
            return false;
        }
        return true;
    }


    public interface Callback {
        boolean onReceive(byte[] bytes);
    }

    public interface AllCallback {
        /**
         * @param what
         * @param bytes
         * @return 是否继续异步接收
         */
        boolean onReceive(short what, byte[] bytes);
    }

    public void setConnectListener(ConnectListener connectListener) {
        this.connectListener = connectListener;
    }


    @Override
    protected void onReceiveName(String deviceName) {
        this.deviceName = deviceName;
    }

    public String getDeviceName() {
        return deviceName;
    }

    public interface ConnectListener {
        /**
         * 断开连接
         */
        void onDisconnect(NovaDevice channel, String ip);

        /**
         * 连接成功
         */
        void onConnected(NovaDevice channel, String ip);

        /**
         * 收到心跳
         *
         * @param channel
         * @param ip
         */
        void onReceivedHeartBeat(NovaDevice channel, String ip);
    }


}
