package nova.traffic.server;

import nova.traffic.server.NovaDevice.ConnectListener;
import nova.traffic.utils.LogUtil;
import nova.traffic.utils.StringUtils;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.util.*;
import java.util.stream.Collectors;

public class ServerChannel extends Channel implements ConnectListener {
    private final String TAG = "ServerChannel ";
    private ServerSocketChannel serverChannel;
    private Selector selector;
    private Map<String, NovaDevice> novaDeviceMap = new HashMap<String, NovaDevice>();
    private volatile boolean isClose = false;
    private ConnectListener connectListener;

    private volatile boolean isStartCheckHeart = false;

    private final Object mapLock = new Object();

    public boolean open(String ip, int port) {

        try {
            //创建一个选择器
            selector = Selector.open();
            isClose = false;
            serverChannel = ServerSocketChannel.open();
            serverChannel.configureBlocking(false);
            serverChannel.socket().bind(new InetSocketAddress(ip, port));
            serverChannel.register(selector, SelectionKey.OP_ACCEPT);
            Thread thread = new Thread(() -> {
                while (!isClose) {
                    try {
                        selector.select();
                        Set<SelectionKey> keys = selector.selectedKeys();
                        Iterator<SelectionKey> iter = keys.iterator();
                        while (iter.hasNext()) {
                            SelectionKey key = iter.next();
                            if (!key.isValid()) {
//									LogUtil.log("key不可用");
                                continue;
                            }
                            synchronized (mapLock) {
                                //                                LogUtil.logPlay("开始时间:" + start);
                                if (key.isAcceptable())  // 新的连接
                                {
                                    ServerSocketChannel ssc = (ServerSocketChannel) key.channel();
                                    SocketChannel channel = ssc.accept();// 得到客户端的SocketChannel
                                    channel.socket().setKeepAlive(true);
                                    String ip1 = channel.socket().getInetAddress().getHostAddress();
                                    int devicePort = channel.socket().getPort();
                                    LogUtil.log("新的链接：" + ip1 + ":" + devicePort);
                                    if (accept(ip1)) {
                                        creatClientChannel(ip1, devicePort, channel);
                                        channel.configureBlocking(false);
                                        channel.register(selector, SelectionKey.OP_READ | SelectionKey.OP_WRITE);
                                    }
                                    //检查无效设备并删除
                                    removeInvalidDevice();
                                } else if (key.isReadable())// 可读
                                {
                                    onReadable(key.channel());
                                } else if (key.isWritable()) {
                                    onWritable(key.channel());
                                }
                                iter.remove(); //处理完事件的要从keys中删去
                            }
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        LogUtil.log(TAG + "socket错误 " + e.toString());
                    }
                }
                LogUtil.log(TAG + "selector 循环跳出");
            });
            thread.setUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
                @Override
                public void uncaughtException(Thread t, Throwable e) {
                    LogUtil.log(TAG + "socket主线程异常" + e.toString());
                }
            });
            thread.start(); //开启一个线程 处理所有请求

            return true;
        } catch (IOException e1) {
            LogUtil.log(TAG + "selector io 异常 " + e1.toString());
            return false;
        }
    }

    protected boolean accept(String ip) {
        return true;
    }

    protected void creatClientChannel(String ip, int devicePort, SocketChannel socketChannel) {
        closeAndRemoveDevice(ip, devicePort);
        NovaDevice device = new NovaDevice(ip, devicePort, socketChannel);
        device.listen(this);
        device.setConnectListener(this);
        addDevice(device);
    }

    private void addDevice(NovaDevice device) {
        String ip = device.ip;
        if (novaDeviceMap != null && ip != null) {
            synchronized (novaDeviceMap) {
                novaDeviceMap.put(deviceKey(ip, device.port), device);
            }
        }
    }

    private void closeAndRemoveDevice(String ip, int port) {
        if (novaDeviceMap != null && ip != null) {
            synchronized (novaDeviceMap) {
                NovaDevice device = novaDeviceMap.get(deviceKey(ip, port));
                if (device != null) {
                    synchronized (device) {
                        device.close();
                    }
                    LogUtil.log("移除ip和端口号重复设备" + ip + ":" + port);
                    novaDeviceMap.remove(deviceKey(ip, port));
                }
            }
        }
    }

    public void close() {
        isClose = true;
        if (selector!=null){
            try {
                selector.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }

        if (serverChannel != null) {
            try {
                serverChannel.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }


        if (novaDeviceMap != null) {
            synchronized (novaDeviceMap) {
                for (NovaDevice device : novaDeviceMap.values()) {
                    synchronized (device) {
                        device.close();
                    }
                }
            }
        }
        LogUtil.log(TAG + "close");
    }

    public void setClientConnectListener(ConnectListener connectListener) {
        this.connectListener = connectListener;
    }


    @Override
    public void onDisconnect(NovaDevice channel, String ip) {
        if (connectListener != null) {
            connectListener.onDisconnect(channel, ip);
        }
    }

    @Override
    public void onConnected(NovaDevice channel, String ip) {
        removeDuplicateDevices(channel);
        if (connectListener != null) {
            connectListener.onConnected(channel, ip);
        }
    }

    @Override
    public void onReceivedHeartBeat(NovaDevice channel, String ip) {
        if (connectListener != null) {
            connectListener.onReceivedHeartBeat(channel, ip);
        }
    }

    /**
     * 公网环境下有可能ip会改变，根据名称移除重复设备
     *
     * @param channel
     */
    private void removeDuplicateDevices(NovaDevice channel) {
        if (channel != null && novaDeviceMap != null) {
            String deviceName = channel.getDeviceName();
            if (deviceName == null) {
                return;
            }
            synchronized (novaDeviceMap) {
                for (String key : novaDeviceMap.keySet()) {
                    NovaDevice device = novaDeviceMap.get(key);
                    if (device != null && device != channel) {
                        if (deviceName.equals(device.getDeviceName())) {
                            LogUtil.log("移除重名设备" + deviceName + " " + key);
                            device.close();
                            novaDeviceMap.remove(key);
                            return;
                        }
                    }
                }
            }
        }
    }

    /**
     * 该方法舍弃，外网环境下，在同一网段下的设备ip一致，导致无法进行多设备同步播放，增加端口号
     * 简易使用 getDeviceByName（name）获取设备
     *
     * @param ip
     * @return
     */
    @Deprecated
    public NovaDevice getDeviceByIp(String ip, int port) {
        if (ip == null || novaDeviceMap == null) {
            return null;
        }
        synchronized (novaDeviceMap) {
            return novaDeviceMap.get(deviceKey(ip, port));
        }
    }

    /**
     * 通过设备名称获取
     *
     * @param name
     * @return
     */
    public NovaDevice getDeviceByName(String name) {
        if (name == null || novaDeviceMap == null) {
            return null;
        }
        synchronized (novaDeviceMap) {
            for (NovaDevice device : novaDeviceMap.values()) {
                if (name.equals(device.getDeviceName())) {
                    return device;
                }
            }
        }
        return null;
    }

    public List<NovaDevice> getAllDevices() {
        if (novaDeviceMap == null) {
            return null;
        }
        synchronized (novaDeviceMap) {
            List<NovaDevice> devices = new ArrayList<NovaDevice>(novaDeviceMap.values());
            //筛选出nova设备，
            List<NovaDevice> collect = devices.stream().filter(novaDevice -> StringUtils.isNotBlank(novaDevice.getDeviceName())).collect(Collectors.toList());
            return collect;
        }
    }

    /**
     * 心跳检查
     *
     * @param heartInterval 检查时间
     * @param timeOut       超时时间
     */
    public synchronized void startCheckHeartbeat(int heartInterval, int timeOut) {
        if (isStartCheckHeart) {
            return;
        }
        if (heartInterval < 30) {
            heartInterval = 30;
        }
        isStartCheckHeart = true;
        long finalHeartInterval = heartInterval * 1000L;
        Thread thread = new Thread(new Runnable() {
            @Override
            public void run() {
                LogUtil.log(TAG + "startCheckHeartbeat");
                while (!isClose) {
                    try {
                        Thread.sleep(finalHeartInterval);
                        //设置范围（心跳30s一次，可通过设置3次心跳为界限）
                        synchronized (mapLock) {
                            check(timeOut * 1000L);
                        }
                    } catch (Exception e) {
                        LogUtil.log(e.toString());
                        Thread.currentThread().interrupt();
                    }
                }
                isStartCheckHeart = false;
                LogUtil.log(TAG + "checkHeartbeat end");
            }

            /**
             * 检查心跳
             */
            private void check(long timeOut) {
                LogUtil.log(TAG + "checkHeartbeat check");
                if (novaDeviceMap != null) {
                    synchronized (novaDeviceMap) {
                        Iterator<NovaDevice> iterator = novaDeviceMap.values().iterator();
                        LogUtil.log("开始检查心跳");
                        long onlineCount = 0;
                        long offlineCount = 0;
                        long total = novaDeviceMap.size();
                        while (iterator.hasNext()) {
                            NovaDevice device = iterator.next();
                            //最后一次心跳时间
                            long lastHeartbeatTime = device.getLastHeartbeatTime();
                            if (System.currentTimeMillis() - lastHeartbeatTime > timeOut) {
                                offlineCount++;
                                LogUtil.log("设备超时，客户端断开 ip:" + device.ip + " name: " + device.getDeviceName());
                                device.onDisconnect();
                                device.close();
                                iterator.remove();
                                LogUtil.log("设备超时，客户端断开 remove");
                            } else {
                                onlineCount++;
                                LogUtil.log("心跳检查，客户端在线 ip:" + device.ip + " name: " + device.getDeviceName());
                            }
                        }
                        LogUtil.log("心跳检查结束，总设备数量：" + total + " 在线数量：" + onlineCount + " 离线数量：" + offlineCount);
                    }
                } else {
                    LogUtil.log(TAG + "checkHeartbeat, novaDeviceMap is null");
                }
            }


        });
        thread.setUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {
            @Override
            public void uncaughtException(Thread t, Throwable e) {
                LogUtil.log(TAG + "心跳检查线程异常" + e.toString());
            }
        });
        thread.start();
    }

    /**
     * 主动检查心跳方法
     *
     * @param heartInterval 心跳检查间隔，最小30s
     */
    public synchronized void startCheckHeartbeat(int heartInterval) {
        startCheckHeartbeat(heartInterval, 90);
    }

    /**
     * 主动检查心跳方法
     */
    public synchronized void startCheckHeartbeat() {
        startCheckHeartbeat(30, 90);
    }

    /**
     * 检查无效设备【在网络环境下，4G环境下设备ip变化，或者其他原因导致的出现设备名称为null的设备，没有实际意义，需要删除掉】
     */
    private void removeInvalidDevice() {
        if (novaDeviceMap != null) {
            synchronized (novaDeviceMap) {
                Iterator<NovaDevice> iterator = novaDeviceMap.values().iterator();
                while (iterator.hasNext()) {
                    NovaDevice device = iterator.next();
                    //1.设备名称为null，2.检查时间需要超过2min接入的设备
                    if (!device.isUseDevice()) {
                        LogUtil.log("非有效设备 ip:" + device.ip + " name: " + device.getDeviceName());
                        device.close();
                        device.onDisconnect();
                        iterator.remove();
                    }
                }

            }
        }
    }
}
