package nova.traffic.utils;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import nova.traffic.been.*;
import nova.traffic.fileserver.FileServer;
import nova.traffic.media.TJRoute;
import nova.traffic.task.FileReceiveTask;
import nova.traffic.utils.contract.ISocket;
import nova.traffic.utils.contract.SocketChannelImpl;
import nova.traffic.utils.contract.SocketImpl;

import java.io.*;
import java.net.*;
import java.nio.channels.SocketChannel;
import java.nio.charset.StandardCharsets;
import java.util.*;

public class NovaTraffic implements NovaTrafficImp {
    private static final int SUCCESS = 1;

    private static final int TEMP_SIZE = 4096;
    private static final int BIG_SIZE = 65535;
    private static final int BIG_SIZE_SCREEN_SHOT = 65535;
    private static final int MAX_FAIL_COUNT = 10;

    /**
     * 默认超时时间
     */
    private static final int DEFAULT_TIMEOUT = 10 * 1000;

    /**
     * socket连接超时时间
     */
    private int connectTimeout = DEFAULT_TIMEOUT;
    /**
     * so超时时间
     */
    private int soTimeout = DEFAULT_TIMEOUT;

    private String ip;
    private int port;
    private String mEncryptKey;
    private int encryptedType = 0;
    private ProtocalParser mProtocalParser;

    private ISocket mSocket;

    /**
     * 初始化通用接口,此方法可以用来公网监听端口建立连接
     *
     * @param socket
     */
    public NovaTraffic(Socket socket) {
        initSocket(socket);
    }

    /**
     * 初始化通用接口
     *
     * @param ip
     * @param port
     */
    public NovaTraffic(String ip, int port) {
        this.ip = ip;
        this.port = port;
    }

    private void initSocket(Socket socket) {
        try {
            Thread.sleep(100);
        } catch (Exception e) {
            LogUtil.log("NovaTraffic:" + e.toString());
            Thread.currentThread().interrupt();
        }
        mSocket = new SocketImpl(socket);
        mProtocalParser = null;
    }

    public NovaTraffic(SocketChannel socketChannel) {
        mSocket = new SocketChannelImpl(socketChannel);
        mProtocalParser = null;
    }

    /**
     * 设置连接超时时间
     *
     * @param connectTimeout
     */
    public void setConnectTimeout(int connectTimeout) {
        this.connectTimeout = connectTimeout;
    }

    /**
     * 设置SO超时时间
     */
    public void setSoTimeout(int soTimeout) {
        this.soTimeout = soTimeout;
    }

    private byte[] getBytes(String str) throws UnsupportedEncodingException {
        return str.getBytes(StandardCharsets.UTF_8);
    }

    /**
     * 获取设备名称
     *
     * @return 设备名称
     */
    public String getDeviceName() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[1];
                bytes[0] = 1;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_DEVICE_NAME_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_DEVICE_NAME_RSP);
                if (recvBuf != null && recvBuf[1] == SUCCESS) {
                    return new String(recvBuf, 2, recvBuf.length - 2);
                }
            } catch (UnknownHostException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }

            return null;
        }
    }

    /**
     * 设置设备名称
     *
     * @param name
     * @return 状态码
     */
    public int setDeviceName(String name) {
        synchronized (this) {
            byte[] nameBytes;
            try {
                nameBytes = getBytes(name);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }
            byte[] bytes = new byte[1 + nameBytes.length];
            bytes[0] = 0;
            System.arraycopy(nameBytes, 0, bytes, 1, nameBytes.length);

            return normalSet(NovaUtils.WHAT_DEVICE_NAME_REQ, NovaUtils.WHAT_DEVICE_NAME_RSP, bytes);
        }
    }

    /**
     * 收到内容 和 发送内容加密
     *
     * @param password
     * @return 是否成功
     */
    public boolean novaTrafficSetPassword(String password) {
        if (password != null && password.getBytes().length == 8) {
            encryptedType = 1;
            mEncryptKey = password;
            return true;
        } else {
            return false;
        }
    }

    /**
     * 收到内容 和 发送内容加密
     *
     * @param password
     * @return 是否成功
     */
    public boolean novaTrafficSetAesPassword(String password) {
        if (password != null && password.getBytes().length == 8) {
            mEncryptKey = Md5Util.MD5(password);
            if (mEncryptKey != null) {
                encryptedType = 2;
                return true;
            }
            return false;
        } else {
            return false;
        }
    }

    public void clearPassword() {
        encryptedType = 0;
        mEncryptKey = null;
    }

    /**
     * 用于客户端短连接建立socket，长连接模式直接赋值
     *
     * @return
     * @throws UnknownHostException
     * @throws IOException
     */
    private ISocket getSocket() throws IOException {
        if (mSocket == null) {
            return createISocket(ip, port);
        } else {
            if (!mSocket.isClosed() && mSocket.isConnected()) {
                return mSocket;
            } else {
                return null;
            }
        }

    }

    /**
     * 创建客户端socket
     *
     * @param ip
     * @param port
     * @return
     * @throws IOException
     */
    protected ISocket createISocket(String ip, int port) throws IOException {
        return new SocketImpl(createClientSocket(ip, port));
    }

    /**
     * 创建客户端socket
     *
     * @param ip
     * @param port
     * @return
     * @throws IOException
     */
    protected Socket createClientSocket(String ip, int port) throws IOException {
        Socket socket = new Socket();
        SocketAddress address = new InetSocketAddress(ip, port);
        try {
            socket.connect(address, connectTimeout);
        } catch (SocketTimeoutException ex) {
            return null;
        }
        socket.setSoTimeout(soTimeout);
        return socket;
    }

    /**
     * 用于客户端短连接关闭socket，长连接模式不关闭
     *
     * @param closeable
     */
    private void closeSocket(Closeable closeable) {
        if (mSocket == null && closeable != null) {
            try {
                closeable.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 用于发送加密
     *
     * @param what
     * @param data
     * @param dataSize
     * @return
     */
    private byte[] makeSendPacket(byte what, byte[] data, int dataSize) {
        if (mEncryptKey != null) {
            return NovaUtils.makeSendPacket(what, data, dataSize, encryptedType, mEncryptKey.getBytes());
        } else {
            return NovaUtils.makeSendPacket(what, data, dataSize);
        }
    }

    /**
     * 用于发送加密
     *
     * @param what
     * @param data
     * @param dataSize
     * @return
     */
    protected byte[] makeSendPacket(short what, byte[] data, int dataSize) {
        int offset = 0;
        if (what > (NovaUtils.WHAT_EXPAND_PARAM_MARK & 0xFF)) {
            byte[] newData = new byte[dataSize + 2];
            NovaUtils.shortToBytes(newData, offset, what);
            offset += 2;
            System.arraycopy(data, 0, newData, offset, dataSize);
            offset += dataSize;
            return makeSendPacket(NovaUtils.WHAT_EXPAND_PARAM_MARK, newData, offset);
        } else {
            return makeSendPacket((byte) what, data, dataSize);
        }

    }

    /**
     * 发送播放文件。 content例子 "[all]\r\n" + "items=2\r\n" + "[item1]\r\n" +
     * "param=30,1,1,1,0,5,1,0,1\r\n" + "txt1=0,0,3,2121,1,8,0,123,0,0,0\r\n" +
     * "txtparam1=0,0\r\n" +
     * "time1=0,0,0,0,30,1,00000000,1616,3,0,1,0,1,1,1,1,1,1,1,1,1\r\n" +
     * "[item2]\r\n" + "param=4,1,1,1,0,5,1,0,1\r\n" +
     * "txt1=0,0,3,1616,1,8,0,456,0,0,0\r\n" + "txtparam1=0,0";
     *
     * @param id      播放文件id 1 代表play001.lst文件
     * @param content 播放内容 utf-8
     * @return
     */
    @Override
    public int sendPlayList(int id, String content) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                String playname = "play" + String.format("%03d", id) + ".lst";
                // 发送文件名
                byte[] bytes = new byte[playname.getBytes().length + 2];
                bytes[0] = (byte) 0xff;
                bytes[1] = (byte) 0xff;
                System.arraycopy(playname.getBytes(), 0, bytes, 2, playname.getBytes().length);

                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_FILENAME_REQ, bytes, bytes.length);
                socket.write(outbuf);
                byte[] inputBuf = new byte[TEMP_SIZE];

                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILENAME_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_NAME_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_FILE_NAME_RECV_ERROR;
                }

                // 发送文件块 1块
                bytes = new byte[getBytes(content).length + 2];
                NovaUtils.shortToBytes(bytes, 0, (short) 1);
                System.arraycopy(getBytes(content), 0, bytes, 2, getBytes(content).length);
                outbuf = makeSendPacket(NovaUtils.WHAT_FILESEND_REQ, bytes, bytes.length);
                socket.write(outbuf);

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[3] != SUCCESS) {
                    return NovaUtils.SEND_FILE_BLOCK_RECV_ERROR;
                }

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_OVER_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_END_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_FILE_BLOCK_RECV_END_ERROR;
                }

                // 指定播放
                byte[] tempbuf = new byte[1];
                tempbuf[0] = (byte) id;
                outbuf = makeSendPacket(NovaUtils.WHAT_PLAYBYLIST_REQ, tempbuf, tempbuf.length);
                socket.write(outbuf);

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_PLAYBYLIST_RSP);
                if (recvBuf == null) {
                    return NovaUtils.PLAY_LIST_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.PLAY_LIST_RECV_ERROR;
                }

                return SUCCESS;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 指定播放列表（和下发文件一起配合使用，sendPlayList为下发播放文件+指定播放组合指令）
     *
     * @param id
     * @return 状态码
     */
    public int playListById(int id) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 指定播放
                byte[] tempbuf = new byte[1];
                tempbuf[0] = (byte) id;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_PLAYBYLIST_REQ, tempbuf, tempbuf.length);
                socket.write(outbuf);

                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_PLAYBYLIST_RSP);
                if (recvBuf == null) {
                    return NovaUtils.PLAY_LIST_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.PLAY_LIST_RECV_ERROR;
                }

                return SUCCESS;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 发送局部更新。 content例子 "[all]\r\n" + "items=2\r\n" + "[item1]\r\n" +
     * "param=30,1,1,1,0,5,1,0,1\r\n" + "txt1=0,0,3,2121,1,8,0,123,0,0,0\r\n" +
     * "txtparam1=0,0\r\n" +
     * "time1=0,0,0,0,30,1,00000000,1616,3,0,1,0,1,1,1,1,1,1,1,1,1\r\n" +
     * "[item2]\r\n" + "param=4,1,1,1,0,5,1,0,1\r\n" +
     * "txt1=0,0,3,1616,1,8,0,456,0,0,0\r\n" + "txtparam1=0,0";
     *
     * @param id      局部更新id
     * @param content 播放内容 utf-8
     * @return
     */
    @Override
    public int sendLocalUpdate(int id, String content) {
        synchronized (this) {
            byte[] bytes = null;
            try {
                bytes = new byte[getBytes(content).length + 2];
                bytes[0] = 1;
                bytes[1] = (byte) id;
                System.arraycopy(getBytes(content), 0, bytes, 2, getBytes(content).length);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }

            return normalSet(NovaUtils.WHAT_REGION_UPDATE_REQ, NovaUtils.WHAT_REGION_UPDATE_RSP, bytes);
        }
    }

    /**
     * 移除某个id的局部更新
     *
     * @param id 移除局部更新id
     * @return 状态码
     */
    public int removeLocalUpdate(int id) {
        synchronized (this) {
            byte[] bytes = new byte[2];
            bytes[0] = 0;
            bytes[1] = (byte) id;
            return normalSet(NovaUtils.WHAT_REGION_UPDATE_REQ, NovaUtils.WHAT_REGION_UPDATE_RSP, bytes);
        }
    }

    /**
     * 发送文件 NovaTraffic.sendFile(new File("d:\\001.jpg"), "001.jpg",
     * "192.168.0.220", 5000);
     *
     * @param file         本地发送文件名
     * @param saveFileName 卡上保存的文件名 utf-8编码
     * @return
     */
    public int sendFile(File file, String saveFileName) {
        synchronized (this) {
            ISocket socket = null;
            try {
                int num = 1;
                int outSize = BIG_SIZE_SCREEN_SHOT;
                byte[] outBuf;
                if (file.isFile()) {
                    socket = getSocket();
                    if (socket == null) {
                        return NovaUtils.SOCKET_ERROR;
                    }

                    // 发送文件名
                    byte[] bytes = new byte[getBytes(saveFileName).length + 2];
                    bytes[0] = (byte) 0xff;
                    bytes[1] = (byte) 0xff;
                    System.arraycopy(getBytes(saveFileName), 0, bytes, 2, getBytes(saveFileName).length);

                    byte[] outbuf = makeSendPacket(NovaUtils.WHAT_FILENAME_REQ, bytes, bytes.length);
                    socket.write(outbuf);
                    byte[] inputBuf = new byte[TEMP_SIZE];

                    // 发送文件名回传.
                    byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILENAME_RSP);
                    if (recvBuf == null) {
                        return NovaUtils.SEND_FILE_NAME_NOT_RECV_ERROR;
                    } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                        return NovaUtils.SEND_FILE_NAME_RECV_ERROR;
                    }

                    Integer result = readFileData(file, socket, num, outSize, inputBuf);
                    if (result != null) {
                        return result;
                    }
                } else {
                    LogUtil.log("文件不存在！");
                    return NovaUtils.FILE_NOT_EXIST_ERROR;
                }

                return SUCCESS;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    private Integer readFileData(File file, ISocket socket, int num, int outSize, byte[] inputBuf) {
        byte[] recvBuf;
        byte[] outBuf;
        FileInputStream fis = null;
        // 以字节流方法读取文件
        try {
            fis = new FileInputStream(file);
            // 设置一个，每次 装载信息的容器
            byte[] buffer = new byte[outSize];
            // 开始读取数据
            int fileLen = 0;// 每次读取到的数据的长度
            int lastFileCopySize = 0;
            while ((fileLen = fis.read(buffer)) != -1) {// len值为-1时，表示没有数据了
                // append方法往sb对象里面添加数据
                outBuf = new byte[fileLen + 2];
                short id = (short) (num++ & 0xffff);
                NovaUtils.shortToBytes(outBuf, 0, id);
                System.arraycopy(buffer, 0, outBuf, 2, fileLen);
                lastFileCopySize = fileLen;
                byte[] buf = makeSendPacket(NovaUtils.WHAT_FILESEND_REQ, outBuf, outBuf.length);
                socket.write(buf);

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[3] != SUCCESS) {
                    return NovaUtils.SEND_FILE_BLOCK_RECV_ERROR;
                }

            }

            // 当发送文件为outSize的整数倍时，发送结束标识
            if (lastFileCopySize == outSize) {
                outBuf = new byte[2];
                short id = (short) (num++ & 0xffff);
                NovaUtils.shortToBytes(outBuf, 0, id);

                byte[] buf = makeSendPacket(NovaUtils.WHAT_FILESEND_REQ, outBuf, outBuf.length);
                socket.write(buf);

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[3] != SUCCESS) {
                    return NovaUtils.SEND_FILE_BLOCK_RECV_ERROR;
                }

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_OVER_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_END_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_FILE_BLOCK_RECV_END_ERROR;
                }
                return SUCCESS;
            }

            recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_OVER_RSP);
            if (recvBuf == null) {
                // 如果没有粘包再来一次
                return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_END_ERROR;
            } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                return NovaUtils.SEND_FILE_BLOCK_RECV_END_ERROR;
            }
            // 输出字符串
        } catch (IOException e) {
            e.printStackTrace();
            return NovaUtils.IO_ERROR;
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    /**
     * 发送文件 NovaTraffic.sendFile(new File("d:\\001.jpg"), "001.jpg",
     * "192.168.0.220", 5000);
     *
     * @param fis          文件输入流
     * @param saveFileName 卡上保存的文件名 utf-8编码
     * @return
     */
    public int sendFile(InputStream fis, String saveFileName) {
        synchronized (this) {
            ISocket socket = null;
            try {
                int num = 1;
                int outSize = BIG_SIZE_SCREEN_SHOT;
                byte[] outBuf;


                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                // 发送文件名
                byte[] bytes = new byte[getBytes(saveFileName).length + 2];
                bytes[0] = (byte) 0xff;
                bytes[1] = (byte) 0xff;
                System.arraycopy(getBytes(saveFileName), 0, bytes, 2, getBytes(saveFileName).length);

                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_FILENAME_REQ, bytes, bytes.length);
                socket.write(outbuf);
                byte[] inputBuf = new byte[TEMP_SIZE];

                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILENAME_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_NAME_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_FILE_NAME_RECV_ERROR;
                }
                Integer result = readFileData(fis, socket, num, outSize, inputBuf);
                if (result != null) {
                    return result;
                }


                return SUCCESS;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    private Integer readFileData(InputStream fis, ISocket socket, int num, int outSize, byte[] inputBuf) {
        byte[] recvBuf;
        byte[] outBuf;
        // 以字节流方法读取文件
        try {
            // 设置一个，每次 装载信息的容器
            byte[] buffer = new byte[outSize];
            // 开始读取数据
            int fileLen = 0;// 每次读取到的数据的长度
            int lastFileCopySize = 0;
            while ((fileLen = fis.read(buffer)) != -1) {// len值为-1时，表示没有数据了
                // append方法往sb对象里面添加数据
                outBuf = new byte[fileLen + 2];
                short id = (short) (num++ & 0xffff);
                NovaUtils.shortToBytes(outBuf, 0, id);
                System.arraycopy(buffer, 0, outBuf, 2, fileLen);
                lastFileCopySize = fileLen;
                byte[] buf = makeSendPacket(NovaUtils.WHAT_FILESEND_REQ, outBuf, outBuf.length);
                socket.write(buf);

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[3] != SUCCESS) {
                    return NovaUtils.SEND_FILE_BLOCK_RECV_ERROR;
                }

            }

            // 当发送文件为outSize的整数倍时，发送结束标识
            if (lastFileCopySize == outSize) {
                outBuf = new byte[2];
                short id = (short) (num++ & 0xffff);
                NovaUtils.shortToBytes(outBuf, 0, id);

                byte[] buf = makeSendPacket(NovaUtils.WHAT_FILESEND_REQ, outBuf, outBuf.length);
                socket.write(buf);

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[3] != SUCCESS) {
                    return NovaUtils.SEND_FILE_BLOCK_RECV_ERROR;
                }

                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_OVER_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_END_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_FILE_BLOCK_RECV_END_ERROR;
                }
                return SUCCESS;
            }

            recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_FILESEND_OVER_RSP);
            if (recvBuf == null) {
                // 如果没有粘包再来一次
                return NovaUtils.SEND_FILE_BLOCK_NOT_RECV_END_ERROR;
            } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                return NovaUtils.SEND_FILE_BLOCK_RECV_END_ERROR;
            }
            // 输出字符串
        } catch (IOException e) {
            e.printStackTrace();
            return NovaUtils.IO_ERROR;
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    /**
     * 设置手动亮度调节
     *
     * @param brightness 0 - 255
     * @return
     */
    public int setBrightness(int brightness) {
        synchronized (this) {
            if (brightness < 0 || brightness > 255) {
                brightness = 125;
            }

            byte[] bytes = new byte[2];
            bytes[0] = 2; // 手动
            bytes[1] = (byte) brightness;
            return normalSet(NovaUtils.WHAT_SCREEN_BRIGHTNESS_MODE_REQ, NovaUtils.WHAT_SCREEN_BRIGHTNESS_MODE_RSP,
                    bytes);

        }
    }

    /**
     * 设置开关屏
     *
     * @param power true开 fasle 关
     * @return 状态码
     */
    public int setPower(boolean power) {
        synchronized (this) {
            byte[] bytes = new byte[1];
            bytes[0] = power ? (byte) 1 : 2;
            return normalSet(NovaUtils.WHAT_SCREEN_POWER_REQ, NovaUtils.WHAT_SCREEN_POWER_RSP, bytes);
        }
    }

    /**
     * 获取屏幕状态
     *
     * @return 状态码
     */
    public int getScreenStatus() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return -1;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_SCREEN_STATUS, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_SCREEN_STATUS);
                if (recvBuf != null && recvBuf.length == 2) {
                    if (recvBuf[1] == 1) {
                        return 1;
                    } else {
                        return 2;
                    }
                }
                return -1;
            } catch (IOException e) {
                e.printStackTrace();
                return -1;
            } finally {
                closeSocket(socket);
            }
        }

    }

    /**
     * 清理文件
     *
     * @param type 0-清理所有媒体文件 1-清理无效媒体文 件 2-清理所有文件 3-清理升级包
     * @return 状态码
     */
    public int cleanUpFiles(int type) {
        synchronized (this) {
            if (type < 0 || type > 3) {
                type = 1;
            }
            byte[] bytes = new byte[1];
            bytes[0] = (byte) type;
            return normalSet(NovaUtils.WHAT_CLEAN_PLAYLIST_REQ, NovaUtils.WHAT_CLEAN_PLAYLIST_RSP, bytes);
        }
    }

    /**
     * 获取监控卡数据
     *
     * @return
     */
    public String getSurveil() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_SURVEIL, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_SURVEIL);
                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 2) {
                    return new String(recvBuf, 1, recvBuf.length - 1);
                }
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }

        }
    }

    /**
     * 设置自动亮度调节
     *
     * @param brightnessItem 必须是8组
     * @return 状态码
     */
    public int setBrightnessAuto(List<BrightnessItem> brightnessItem) {
        synchronized (this) {
            int brightSize = 8;
            if (brightnessItem == null || brightnessItem.size() != brightSize) {
                return NovaUtils.PARAM_ERROR;
            }

            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                // 设置自动
                byte[] bytes = new byte[2];
                bytes[0] = 1; // 自动
                bytes[1] = 0;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SCREEN_BRIGHTNESS_MODE_REQ, bytes, bytes.length);
                socket.write(outbuf);

                bytes = new byte[24];
                for (int i = 0; i < brightSize; i++) {
                    bytes[i * 3] = (byte) (0xff & brightnessItem.get(i).getEnvironment());
                    bytes[i * 3 + 1] = (byte) (0xff & brightnessItem.get(i).getScreen());
                    bytes[i * 3 + 2] = 0;
                }

                outbuf = makeSendPacket(NovaUtils.WHAT_SCREEN_BRIGHTNESS_SET_AUTO_MODE_PARAM_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf,
                        NovaUtils.WHAT_SCREEN_BRIGHTNESS_SET_AUTO_MODE_PARAM_RSP);

                if (recvBuf == null) {
                    return NovaUtils.SEND_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_RECV_ERROR;
                }

                return SUCCESS;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 获取自动亮度调节
     *
     * @return 状态码
     */
    public List<BrightnessItem> getBrightnessAuto() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                // 设置自动
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SCREEN_BRIGHTNESS_GET_AUTO_MODE_PARAM_REQ, bytes,
                        bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf,
                        NovaUtils.WHAT_SCREEN_BRIGHTNESS_GET_AUTO_MODE_PARAM_RSP);

                if (recvBuf == null || recvBuf.length != 25) {
                    return null;
                }

                List<BrightnessItem> brightnessItem = new ArrayList<>();
                for (int i = 0; i < 8; i++) {
                    BrightnessItem item = new BrightnessItem(0xff & recvBuf[1 + i * 3], 0xff & recvBuf[2 + i * 3]);
                    brightnessItem.add(item);
                }
                return brightnessItem;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 控制本板电源和屏幕黑屏
     *
     * @return 状态码
     */
    public int setBoardPowerAndScreenPower(boolean power) {
        synchronized (this) {
            byte[] bytes = new byte[1];
            bytes[0] = power ? (byte) 1 : 2;
            return normalSet(NovaUtils.WHAT_SCREEN_MY_BOARD_POWER_AND_POWER_REQ,
                    NovaUtils.WHAT_SCREEN_MY_BOARD_POWER_AND_POWER_RSP, bytes);
        }
    }

    /**
     * 控制本板电源
     *
     * @return 状态码
     */
    public int setBoardPower(boolean power) {
        synchronized (this) {
            byte[] bytes = new byte[1];
            bytes[0] = power ? (byte) 1 : 2;
            return normalSet(NovaUtils.WHAT_SCREEN_MY_BOARD_POWER_REQ, NovaUtils.WHAT_SCREEN_MY_BOARD_POWER_RSP, bytes);
        }
    }

    /**
     * 控制多功能卡电源
     *
     * @param cardIndex 0-255 多功能卡位置索引
     * @param roadIndex 0-7 电源路数索引
     * @param power
     * @return 状态码
     */
    public int setMultiFunctionCardPower(int cardIndex, int roadIndex, boolean power) {
        synchronized (this) {
            byte[] bytes = new byte[3];
            bytes[0] = (byte) (0xff & cardIndex);
            bytes[1] = (byte) (0xff & roadIndex);
            bytes[2] = power ? (byte) 1 : 2;
            return normalSet(NovaUtils.WHAT_FUNCTION_CARD_POWER_REQ, NovaUtils.WHAT_FUNCTION_CARD_POWER_RSP, bytes);
        }
    }

    /**
     * 获取控制卡状态
     *
     * @return
     */
    @SuppressWarnings("deprecation")
    public DeviceType getDeviceType() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_DEVICE_TYPE_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_DEVICE_TYPE_RSP);
                if (recvBuf == null || recvBuf.length != 19) {
                    return null;
                }

                DeviceType mDeviceType = new DeviceType();

                int day = recvBuf[4];
                int month = recvBuf[3] - 1;
                int year = (short) (0xffff & NovaUtils.bytesToShort(recvBuf, 1)) - 1900;
                int second = recvBuf[7];
                int minute = recvBuf[6];
                int hour = recvBuf[5];
                mDeviceType.setDate(new Date(year, month, day, hour, minute, second));

                mDeviceType.setBoxDoorOpen(recvBuf[8] == 1);
                mDeviceType.setScreenPowerOpen(recvBuf[9] == 1);
                int sign = recvBuf[11] == 1 ? 1 : -1;
                mDeviceType.setTemperature(recvBuf[12] * sign);
                mDeviceType.setInputSourceInside(recvBuf[13] == 1);
                mDeviceType.setEnvironmentBrightness(0xff & recvBuf[16]);
                mDeviceType.setBrightnessType(recvBuf[17]);
                mDeviceType.setScreenBrightness(0xff & recvBuf[18]);

                return mDeviceType;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置时间
     *
     * @param date
     * @return 状态码
     */
    @SuppressWarnings("deprecation")
    public int setTime(Date date) {
        synchronized (this) {
            byte[] bytes = new byte[7];
            NovaUtils.shortToBytes(bytes, 0, (short) (0xffff & date.getYear() + 1900));
            bytes[2] = (byte) (0xff & (date.getMonth() + 1));
            bytes[3] = (byte) (0xff & date.getDate());
            bytes[4] = (byte) (0xff & date.getHours());
            bytes[5] = (byte) (0xff & date.getMinutes());
            bytes[6] = (byte) (0xff & date.getSeconds());
            return normalSet(NovaUtils.WHAT_DEVICE_SET_TIME_REQ, NovaUtils.WHAT_DEVICE_SET_TIME_RSP, bytes);

        }
    }

    /**
     * 重启设备
     *
     * @return 状态码
     */
    public int restartDevice() {
        synchronized (this) {
            byte[] bytes = new byte[0];
            return normalSet(NovaUtils.WHAT_DEVICE_REBOOT_REQ, NovaUtils.WHAT_DEVICE_REBOOT_RSP, bytes);

        }
    }

    /**
     * 恢复ip到出厂的192.168.0.220
     */
    public void resetDeviceIpToDefault() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_DEVICE_RESET_IP_REQ, bytes, bytes.length);
                socket.write(outbuf);
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }

        }
    }

    /**
     * 获取控制卡版本
     *
     * @return 状态码
     */
    public String getDeviceVersion() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_QUERY_VERSIONINFO_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_QUERY_VERSIONINFO_RSP);
                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 2) {
                    return new String(recvBuf, 1, recvBuf.length - 1);
                }
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }

        }
    }

    /**
     * 设置屏体基本参数
     */
    public void setDeviceNetBasicParam(DeviceNetBasicParam mDeviceBasicParam) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return;
                }
                byte[] bytes = new byte[26];
                NovaUtils.normalIntToBytes(bytes, 2, NovaUtils.ipToInt(mDeviceBasicParam.getDeviceIp()));
                NovaUtils.shortToBytes(bytes, 6, (short) (0xffff & mDeviceBasicParam.getPort()));
                NovaUtils.normalIntToBytes(bytes, 8, NovaUtils.ipToInt(mDeviceBasicParam.getMark()));
                NovaUtils.normalIntToBytes(bytes, 12, NovaUtils.ipToInt(mDeviceBasicParam.getGate()));
                NovaUtils.normalIntToBytes(bytes, 16, NovaUtils.ipToInt(mDeviceBasicParam.getServiceIp()));
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SET_DEVICE_IP_REQ, bytes, bytes.length);
                socket.write(outbuf);
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置设备加密
     *
     * @param mDeviceSecretParam
     * @return 状态码
     */
    public int setDeviceSecretParam(DeviceSecretParam mDeviceSecretParam) {
        synchronized (this) {
            int offset = 0;
            byte[] bytes = new byte[TEMP_SIZE];
            bytes[offset++] = (byte) mDeviceSecretParam.getEncodeType();
            bytes[offset++] = mDeviceSecretParam.isMD5Open() ? (byte) 1 : 0;
            String password = mDeviceSecretParam.getPassword();
            try {
                if (password != null && getBytes(password).length > 0 && getBytes(password).length <= 16) {
                    System.arraycopy(password.getBytes(), 0, bytes, offset, getBytes(password).length);
                    offset += getBytes(password).length;
                } else {
                    return NovaUtils.PARAM_ERROR;
                }

                bytes[offset++] = (byte) 0x91;
                bytes[offset++] = 0x21;

                String newPassword = mDeviceSecretParam.getNewPassword();
                String oldPassword = mDeviceSecretParam.getOldPassword();
                if (mDeviceSecretParam.getEncodeType() == 0) {
                    LogUtil.log("mDeviceSecretParam.getEncodeType():" + mDeviceSecretParam.getEncodeType());
                } else if (mDeviceSecretParam.getEncodeType() != 0 && newPassword != null && newPassword.getBytes().length == 8) {
                    System.arraycopy(newPassword.getBytes(), 0, bytes, offset, 8);
                    offset += 8;
                } else {
                    return NovaUtils.PARAM_ERROR;
                }
                if (oldPassword != null && oldPassword.length() > 0) {
                    bytes[offset++] = (byte) 0x91;
                    bytes[offset++] = 0x21;
                    if (oldPassword.length() == 8) {
                        System.arraycopy(oldPassword.getBytes(), 0, bytes, offset, 8);
                        offset += 8;
                    } else {
                        return NovaUtils.PARAM_ERROR;
                    }
                }
                byte[] out = new byte[offset];
                System.arraycopy(bytes, 0, out, 0, offset);
                int ret = normalSet(NovaUtils.WHAT_SECRET_DES_MD5_REQ, NovaUtils.WHAT_SECRET_DES_MD5_RSP, out);
                if (ret == SUCCESS) {
                    mProtocalParser = null;
                }
                return ret;
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }
        }
    }

    /**
     * 升级文件
     *
     * @param path utf8编码
     * @return 状态码
     */
    public int updateFileName(String path) {
        synchronized (this) {
            if (path == null) {
                return NovaUtils.PARAM_ERROR;
            }

            try {
                return normalSet(NovaUtils.WHAT_SYSTEM_UPDATE_REQ, NovaUtils.WHAT_SYSTEM_UPDATE_RSP, getBytes(path));
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }

        }
    }

    /**
     * 移除app
     *
     * @param pageName
     * @return 状态码
     */
    public int removeAppPageName(String pageName) {
        synchronized (this) {
            if (pageName == null) {
                return NovaUtils.PARAM_ERROR;
            }

            try {
                return normalSet(NovaUtils.WHAT_APP_REMOVE, NovaUtils.WHAT_APP_REMOVE, getBytes(pageName));
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }

        }
    }

    /**
     * 获取屏幕尺寸（宽高）
     *
     * @return DeviceSize
     */
    public DeviceSize getDeviceSize() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SCREEN_WIDTH_HEIGHT_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_SCREEN_WIDTH_HEIGHT_RSP);
                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length != 5) {
                    return null;
                }

                int width = NovaUtils.bytesToShort(recvBuf, 1);
                int height = NovaUtils.bytesToShort(recvBuf, 3);
                return new DeviceSize(width, height);
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置音量0-100
     *
     * @param volume 0 -100
     * @return 状态码
     */
    public int setDeviceVolume(int volume) {
        synchronized (this) {
            byte[] bytes = new byte[1];
            bytes[0] = (byte) (0xff & volume);
            return normalSet(NovaUtils.WHAT_SET_VOLUME_PARAM_REQ, NovaUtils.WHAT_SET_VOLUME_PARAM_RSP, bytes);
        }
    }

    /**
     * 获取音量
     *
     * @return volume 0 -100
     */
    public int getDeviceVolume() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return -1;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_VOLUME_PARAM, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_VOLUME_PARAM);
                if (recvBuf == null) {
                    return -1;
                } else if (recvBuf.length >= 2) {
                    return recvBuf[1];
                }
                return -1;
            } catch (IOException e) {
                e.printStackTrace();
                return -1;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 获取设备空间
     *
     * @return DeviceMemorySize
     */
    public DeviceMemorySize getDeviceMemorySize() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_SDCARD_SIZE_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_SDCARD_SIZE_RSP);
                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length != 9) {
                    return null;
                }

                int totalSize = NovaUtils.bytesToInt(recvBuf, 1);
                int availableSize = NovaUtils.bytesToInt(recvBuf, 5);
                return new DeviceMemorySize(totalSize, availableSize);
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置时区
     *
     * @param timeZoneParam
     * @return 状态码
     */
    public int setTimeAndTimeZone(TimeZoneParam timeZoneParam) {
        synchronized (this) {
            String outStr = Long.toString(timeZoneParam.getUtcTime()) + "," + timeZoneParam.getTimeZoneId() + ","
                    + timeZoneParam.getGmt();
            try {
                return normalSet(NovaUtils.SET_TIME_AND_ZONE, NovaUtils.SET_TIME_AND_ZONE, getBytes(outStr));
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }

        }
    }

    /**
     * 获取时区
     *
     * @return TimeZoneParam
     */
    public TimeZoneParam getTimeAndTimeZone() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.GET_TIME_AND_ZONE, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.GET_TIME_AND_ZONE);
                if (recvBuf == null) {
                    return null;
                }

                String temp = new String(recvBuf, 1, recvBuf.length - 1);
                String[] listStr = temp.split(",");
                Long utcTimeMillis = Long.valueOf(listStr[0]);
                return new TimeZoneParam(utcTimeMillis, listStr[1], listStr[2]);
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置ntp对时参数
     *
     * @param deviceNTPParam
     * @return 状态码
     */
    public int setDeviceNTPParam(DeviceNTPParam deviceNTPParam) {
        synchronized (this) {
            if (deviceNTPParam.getAddr() == null) {
                deviceNTPParam.setAddr("");
            }

            byte[] bytes;
            try {
                bytes = new byte[1 + getBytes(deviceNTPParam.getAddr()).length];
                bytes[0] = deviceNTPParam.isOpen() ? (byte) 1 : 0;
                System.arraycopy(getBytes(deviceNTPParam.getAddr()), 0, bytes, 1,
                        getBytes(deviceNTPParam.getAddr()).length);
                return normalSet(NovaUtils.SET_NTP_PARAM, NovaUtils.SET_NTP_PARAM, bytes);
            } catch (UnsupportedEncodingException e1) {
                e1.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }
        }
    }

    /**
     * 获取ntp对时参数
     *
     * @return DeviceNTPParam
     */
    public DeviceNTPParam getDeviceNTPParam() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.GET_NTP_PARAM, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.GET_NTP_PARAM);
                if (recvBuf == null) {
                    return null;
                }

                boolean isOpen = recvBuf[1] == 1;

                return new DeviceNTPParam(isOpen, new String(recvBuf, 2, recvBuf.length - 2, StandardCharsets.UTF_8));
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置屏体关闭温度，高于此温度关屏
     *
     * @param value 系统最小值为 70 0 表示不进行关屏处理
     * @return 状态码
     */
    public int setCloseScreenTemperature(int value) {
        synchronized (this) {
            byte[] bytes = new byte[6];
            if (value < 0 || value > 150) {
                return NovaUtils.PARAM_ERROR;
            }
            bytes[5] = (byte) (0xff & value);
            return normalSet(NovaUtils.WHAT_SET_ENVIRONMENT_ALARM_REQ, NovaUtils.WHAT_SET_ENVIRONMENT_ALARM_RSP, bytes);
        }
    }

    /**
     * 获取 屏体关闭温度
     *
     * @return 状态码
     */
    public int getCloseScreenTemperature() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_ENVIRONMENT_ALARM_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_ENVIRONMENT_ALARM_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_NOT_RECV_ERROR;
                }

                return recvBuf[6];
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置虚连接播放文件
     *
     * @param isOpen     false-不开启 true-开启虚连接检测
     * @param secondTime 单位秒，多长时间无数据交互开启虚连接
     * @param id         播放文件id 1 代表play001.lst文件
     * @return 状态码
     */
    public int setVirtualConnection(boolean isOpen, int secondTime, int id) {
        synchronized (this) {
            byte[] bytes = new byte[4];
            bytes[0] = isOpen ? (byte) 1 : 0;

            if (secondTime < 60) {
                secondTime = 60;
            }
            NovaUtils.shortToBytes(bytes, 1, (short) (0xffff & secondTime));
            bytes[3] = (byte) (0xff & id);
            return normalSet(NovaUtils.WHAT_VIRTUAL_CONNECT_PARAM_REQ, NovaUtils.WHAT_VIRTUAL_CONNECT_PARAM_RSP, bytes);
        }
    }

    /**
     * 发送ftp下载文件地址
     *
     * @param url ftp://user:password@192.168.0.108:21/play001.lst
     * @return 状态码
     */
    public int sendFtpDownloadaddress(String url) {
        synchronized (this) {
            if (url == null || url.length() == 0) {
                return NovaUtils.PARAM_ERROR;
            }
            try {
                return normalSet(NovaUtils.WHAT_SET_FTP_DOWNLOAD_REQ, NovaUtils.WHAT_SET_FTP_DOWNLOAD_RSP,
                        getBytes(url));
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }
        }
    }

    /**
     * 获取当前播放内容
     *
     * @return DeviceNowPlayItem
     */
    public DeviceNowPlayItem getNowPlayContent() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_PLAYING_ITEM_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_PLAYING_ITEM_RSP);

                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 4) {
                    boolean isOpen = recvBuf[1] == 1;
                    String content = new String(recvBuf, 4, recvBuf.length - 4, StandardCharsets.UTF_8);
                    return new DeviceNowPlayItem(isOpen, recvBuf[3], content);
                }
                return null;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 获取当前播放全部内容
     *
     * @return DeviceNowPlayList
     */
    public DeviceNowPlayList getNowPlayAllContent() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_PLAYING_PLAYLIST_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_PLAYING_PLAYLIST_RSP);

                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 2) {
                    String content = new String(recvBuf, 2, recvBuf.length - 2, StandardCharsets.UTF_8);
                    return new DeviceNowPlayList(recvBuf[1], content);
                }
                return null;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 获取对应播放全部内容
     *
     * @param id 对应播放列表
     * @return PlayListContent
     */
    public String getPlayListContent(int id) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[1];
                bytes[0] = (byte) id;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_PLAYLIST_BY_ID, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_PLAYLIST_BY_ID);

                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 2) {
                    return new String(recvBuf, 1, recvBuf.length - 1, StandardCharsets.UTF_8);
                }
                return null;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 获取当前播放截图
     *
     * @param filePath 截图文件名
     * @return 状态码
     */
    public int getDeviceScreenshot(String filePath) {
        synchronized (this) {
            File file = new File(filePath);
            if (!FileUtils.createEmptyFile(file)) {
                return NovaUtils.PARAM_ERROR;
            }

            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }
                byte[] bytes = new byte[2];
                NovaUtils.shortToBytes(bytes, 0, (short) (0xffff & BIG_SIZE));
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SCREENSHOT_REQ, bytes, bytes.length);
                socket.write(outbuf);

                while (true) {
                    byte[] inputBuf = new byte[BIG_SIZE * 2];
                    byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_SCREENSHOT_RSP);
                    if (recvBuf == null) {
                        return NovaUtils.IO_ERROR;
                    }
                    FileUtils.writeFileAppend(file, recvBuf, 3, recvBuf.length - 3);
                    if (recvBuf.length - 3 < BIG_SIZE) {
                        return SUCCESS;
                    }
                }
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 获取当前播放截图
     * 通过base64格式返回
     *
     * @return 截图的Base64
     */
    public String getBase64Screenshot(File file) {
        synchronized (NovaTraffic.class) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[2];
                NovaUtils.shortToBytes(bytes, 0, (short) (0xffff & BIG_SIZE));
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SCREENSHOT_REQ, bytes, bytes.length);
                LogUtil.log("outbuf：" + outbuf.length);
                socket.write(outbuf);
                byte[] fileByte = null;
                while (true) {
                    byte[] inputBuf = new byte[BIG_SIZE * 2];
                    byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_SCREENSHOT_RSP);
                    if (recvBuf == null) {
                        break;
                    }
                    LogUtil.log("recvBuf：" + recvBuf.length);
                    if (fileByte == null) {
                        fileByte = new byte[recvBuf.length - 3];
                        System.arraycopy(recvBuf, 3, fileByte, 0, recvBuf.length - 3);
                    } else {
                        int length = fileByte.length;
                        byte[] arr = Arrays.copyOf(fileByte, length + recvBuf.length - 3);
                        System.arraycopy(recvBuf, 3, arr, length, recvBuf.length - 3);
                        fileByte = new byte[arr.length];
                        System.arraycopy(arr, 0, fileByte, 0, fileByte.length);
                    }
                    if (recvBuf.length - 3 < BIG_SIZE) {
                        break;
                    }
                }
                LogUtil.log("fileByte：" + fileByte.length);
                String s = Base64.getEncoder().encodeToString(fileByte);
                byte[] decode = Base64.getDecoder().decode(s);
                FileUtils.writeFileAppend(file, decode, 0, decode.length);
                return s;
            } catch (Exception e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置定时开屏列表
     *
     * @param list 为null 或 0 清除操作 ；新表覆盖旧表
     * @return 状态码
     */
    @SuppressWarnings("deprecation")
    public int setDevicePowerByTimeList(List<DevicePowerByTimeParam> list) {
        synchronized (this) {
            byte[] bytes;
            if (list == null || list.isEmpty()) {
                bytes = new byte[1];
                bytes[0] = 0;
            } else {
                int itemSize = 14;
                bytes = new byte[1 + list.size() * itemSize];
                bytes[0] = (byte) list.size();
                for (int i = 0; i < list.size(); i++) {
                    bytes[1 + i * itemSize] = (byte) (list.get(i).getStart().getDate());
                    bytes[2 + i * itemSize] = (byte) (list.get(i).getStart().getMonth() + 1);
                    NovaUtils.shortToBytes(bytes, 3 + i * itemSize, (short) (list.get(i).getStart().getYear() + 1900));
                    bytes[5 + i * itemSize] = (byte) list.get(i).getStart().getSeconds();
                    bytes[6 + i * itemSize] = (byte) list.get(i).getStart().getMinutes();
                    bytes[7 + i * itemSize] = (byte) list.get(i).getStart().getHours();

                    bytes[8 + i * itemSize] = (byte) (list.get(i).getEnd().getDate());
                    bytes[9 + i * itemSize] = (byte) (list.get(i).getEnd().getMonth() + 1);
                    NovaUtils.shortToBytes(bytes, 10 + i * itemSize, (short) (list.get(i).getEnd().getYear() + 1900));
                    bytes[12 + i * itemSize] = (byte) list.get(i).getEnd().getSeconds();
                    bytes[13 + i * itemSize] = (byte) list.get(i).getEnd().getMinutes();
                    bytes[14 + i * itemSize] = (byte) list.get(i).getEnd().getHours();
                }
            }
            return normalSet(NovaUtils.WHAT_SCREEN_OPEN_CLOSE_SET_TIMING_PARAM_REQ,
                    NovaUtils.WHAT_SCREEN_OPEN_CLOSE_SET_TIMING_PARAM_RSP, bytes);
        }
    }

    /**
     * 获取定时开屏列表
     *
     * @return
     */
    @SuppressWarnings("deprecation")
    public List<DevicePowerByTimeParam> getDevicePowerByTimeList() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.GET_SCREEN_OPEN_CLOSE_SET_TIMING_PARAM_INFO, bytes,
                        bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf,
                        NovaUtils.GET_SCREEN_OPEN_CLOSE_SET_TIMING_PARAM_INFO);

                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 2) {
                    int length = recvBuf[1];
                    int itemSize = 14;
                    List<DevicePowerByTimeParam> list = new ArrayList<>();
                    for (int i = 0; i < length; i++) {
                        DevicePowerByTimeParam param = new DevicePowerByTimeParam();
                        int day = recvBuf[2 + i * itemSize];
                        int month = recvBuf[3 + i * itemSize] - 1;
                        int year = NovaUtils.bytesToShort(recvBuf, 4 + i * itemSize) - 1900;
                        int second = recvBuf[6 + i * itemSize];
                        int minute = recvBuf[7 + i * itemSize];
                        int hour = recvBuf[8 + i * itemSize];
                        param.setStart(new Date(year, month, day, hour, minute, second));

                        day = recvBuf[9 + i * itemSize];
                        month = recvBuf[10 + i * itemSize] - 1;
                        year = NovaUtils.bytesToShort(recvBuf, 11 + i * itemSize) - 1900;
                        second = recvBuf[13 + i * itemSize];
                        minute = recvBuf[14 + i * itemSize];
                        hour = recvBuf[15 + i * itemSize];
                        param.setEnd(new Date(year, month, day, hour, minute, second));
                        list.add(param);
                    }
                    return list;
                }
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置定时播放列表
     *
     * @param list
     * @return 状态码
     */
    @SuppressWarnings("deprecation")
    public int setPlayByTimeList(List<PlayByTimeParam> list) {
        synchronized (this) {
            byte[] bytes;
            if (list == null || list.isEmpty()) {
                bytes = new byte[1];
                bytes[0] = 0;
            } else {
                int itemSize = 16;
                bytes = new byte[1 + list.size() * itemSize];
                bytes[0] = (byte) list.size();
                for (int i = 0; i < list.size(); i++) {
                    bytes[1 + i * itemSize] = (byte) list.get(i).getStart().getDate();
                    bytes[2 + i * itemSize] = (byte) (list.get(i).getStart().getMonth() + 1);
                    NovaUtils.shortToBytes(bytes, 3 + i * itemSize, (short) (list.get(i).getStart().getYear() + 1900));
                    bytes[5 + i * itemSize] = (byte) list.get(i).getStart().getSeconds();
                    bytes[6 + i * itemSize] = (byte) list.get(i).getStart().getMinutes();
                    bytes[7 + i * itemSize] = (byte) list.get(i).getStart().getHours();

                    bytes[8 + i * itemSize] = (byte) list.get(i).getEnd().getDate();
                    bytes[9 + i * itemSize] = (byte) (list.get(i).getEnd().getMonth() + 1);
                    NovaUtils.shortToBytes(bytes, 10 + i * itemSize, (short) (list.get(i).getEnd().getYear() + 1900));
                    bytes[12 + i * itemSize] = (byte) list.get(i).getEnd().getSeconds();
                    bytes[13 + i * itemSize] = (byte) list.get(i).getEnd().getMinutes();
                    bytes[14 + i * itemSize] = (byte) list.get(i).getEnd().getHours();

                    bytes[15 + i * itemSize] = (byte) list.get(i).getId();
                }
            }
            return normalSet(NovaUtils.WHAT_SET_TIMING_PLAYLIST_PARAM_REQ, NovaUtils.WHAT_SET_TIMING_PLAYLIST_PARAM_RSP,
                    bytes);
        }
    }

    /**
     * 获取定时播放列表
     *
     * @return
     */
    @SuppressWarnings("deprecation")
    public List<PlayByTimeParam> getPlayByTimeList() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.GET_SCREEN_TIMING_PLAYLIST_PARAM_INFO, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf,
                        NovaUtils.GET_SCREEN_TIMING_PLAYLIST_PARAM_INFO);

                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 2) {
                    int length = recvBuf[1];
                    int itemSize = 16;
                    List<PlayByTimeParam> list = new ArrayList<>();
                    for (int i = 0; i < length; i++) {
                        PlayByTimeParam param = new PlayByTimeParam();
                        int day = recvBuf[2 + i * itemSize];
                        int month = recvBuf[3 + i * itemSize] - 1;
                        int year = NovaUtils.bytesToShort(recvBuf, 4 + i * itemSize) - 1900;
                        int second = recvBuf[6 + i * itemSize];
                        int minute = recvBuf[7 + i * itemSize];
                        int hour = recvBuf[8 + i * itemSize];
                        param.setStart(new Date(year, month, day, hour, minute, second));

                        day = recvBuf[9 + i * itemSize];
                        month = recvBuf[10 + i * itemSize] - 1;
                        year = NovaUtils.bytesToShort(recvBuf, 11 + i * itemSize) - 1900;
                        second = recvBuf[13 + i * itemSize];
                        minute = recvBuf[14 + i * itemSize];
                        hour = recvBuf[15 + i * itemSize];
                        param.setEnd(new Date(year, month, day, hour, minute, second));

                        param.setId(recvBuf[16 + i * itemSize]);
                        list.add(param);
                    }
                    return list;
                }
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置定时亮度列表
     *
     * @param list
     * @return 状态码
     */
    @SuppressWarnings("deprecation")
    public int setBrightnessByTimeList(List<BrightnessByTimeParam> list) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                // 设置定时
                byte[] bytes = new byte[2];
                bytes[0] = 3; // 定时
                bytes[1] = 0;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SCREEN_BRIGHTNESS_MODE_REQ, bytes, bytes.length);
                socket.write(outbuf);
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }

            byte[] bytes = null;
            if (list == null || list.isEmpty()) {
                bytes = new byte[1];
                bytes[0] = 0;
            } else {
                int itemSize = 15;
                bytes = new byte[1 + list.size() * itemSize];
                bytes[0] = (byte) list.size();
                for (int i = 0; i < list.size(); i++) {
                    bytes[1 + i * itemSize] = (byte) list.get(i).getStart().getDate();
                    bytes[2 + i * itemSize] = (byte) (list.get(i).getStart().getMonth() + 1);
                    NovaUtils.shortToBytes(bytes, 3 + i * itemSize, (short) (list.get(i).getStart().getYear() + 1900));
                    bytes[5 + i * itemSize] = (byte) list.get(i).getStart().getSeconds();
                    bytes[6 + i * itemSize] = (byte) list.get(i).getStart().getMinutes();
                    bytes[7 + i * itemSize] = (byte) list.get(i).getStart().getHours();

                    bytes[8 + i * itemSize] = (byte) list.get(i).getEnd().getDate();
                    bytes[9 + i * itemSize] = (byte) (list.get(i).getEnd().getMonth() + 1);
                    NovaUtils.shortToBytes(bytes, 10 + i * itemSize, (short) (list.get(i).getEnd().getYear() + 1900));
                    bytes[12 + i * itemSize] = (byte) list.get(i).getEnd().getSeconds();
                    bytes[13 + i * itemSize] = (byte) list.get(i).getEnd().getMinutes();
                    bytes[14 + i * itemSize] = (byte) list.get(i).getEnd().getHours();

                    bytes[15 + i * itemSize] = (byte) list.get(i).getBrightness();
                }
            }
            return normalSet(NovaUtils.WHAT_SCREEN_BRIGHTNESS_SET_TIMING_MODE_PARAM_REQ,
                    NovaUtils.WHAT_SCREEN_BRIGHTNESS_SET_TIMING_MODE_PARAM_RSP, bytes);

        }
    }

    /**
     * 获取定时亮度列表
     *
     * @return
     */
    @SuppressWarnings("deprecation")
    public List<BrightnessByTimeParam> getBrightnessByTimeList() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.GET_SCREEN_BRIGHTNESS_SET_TIMING_MODE_PARAM_INFO, bytes,
                        bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf,
                        NovaUtils.GET_SCREEN_BRIGHTNESS_SET_TIMING_MODE_PARAM_INFO);

                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 2) {
                    int itemSize = 15;
                    int length = recvBuf[1];
                    List<BrightnessByTimeParam> list = new ArrayList<>();
                    for (int i = 0; i < length; i++) {
                        BrightnessByTimeParam param = new BrightnessByTimeParam();
                        int day = recvBuf[2 + i * itemSize];
                        int month = recvBuf[3 + i * itemSize] - 1;
                        int year = NovaUtils.bytesToShort(recvBuf, 4 + i * itemSize) - 1900;
                        int second = recvBuf[6 + i * itemSize];
                        int minute = recvBuf[7 + i * itemSize];
                        int hour = recvBuf[8 + i * itemSize];
                        param.setStart(new Date(year, month, day, hour, minute, second));

                        day = recvBuf[9 + i * itemSize];
                        month = recvBuf[10 + i * itemSize] - 1;
                        year = NovaUtils.bytesToShort(recvBuf, 11 + i * itemSize) - 1900;
                        second = recvBuf[13 + i * itemSize];
                        minute = recvBuf[14 + i * itemSize];
                        hour = recvBuf[15 + i * itemSize];
                        param.setEnd(new Date(year, month, day, hour, minute, second));

                        param.setBrightness(recvBuf[16 + i * itemSize]);
                        list.add(param);
                    }
                    return list;
                }
                return null;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    public int setTimingVolumeParam(List<VolumeByTimeParam> list) {
        synchronized (this) {
            byte[] bytes;
            if (list == null || list.size() == 0) {
                bytes = new byte[1];
                bytes[0] = 0;
            } else {
                int itemSize = 15;
                bytes = new byte[1 + list.size() * itemSize];
                bytes[0] = (byte) list.size();
                for (int i = 0; i < list.size(); i++) {
                    bytes[1 + i * itemSize] = (byte) list.get(i).getStart().getDate();
                    bytes[2 + i * itemSize] = (byte) (list.get(i).getStart().getMonth() + 1);
                    NovaUtils.shortToBytes(bytes, 3 + i * itemSize, (short) (list.get(i).getStart().getYear() + 1900));
                    bytes[5 + i * itemSize] = (byte) list.get(i).getStart().getSeconds();
                    bytes[6 + i * itemSize] = (byte) list.get(i).getStart().getMinutes();
                    bytes[7 + i * itemSize] = (byte) list.get(i).getStart().getHours();

                    bytes[8 + i * itemSize] = (byte) list.get(i).getEnd().getDate();
                    bytes[9 + i * itemSize] = (byte) (list.get(i).getEnd().getMonth() + 1);
                    NovaUtils.shortToBytes(bytes, 10 + i * itemSize, (short) (list.get(i).getEnd().getYear() + 1900));
                    bytes[12 + i * itemSize] = (byte) list.get(i).getEnd().getSeconds();
                    bytes[13 + i * itemSize] = (byte) list.get(i).getEnd().getMinutes();
                    bytes[14 + i * itemSize] = (byte) list.get(i).getEnd().getHours();

                    bytes[15 + i * itemSize] = (byte) list.get(i).getVolume();
                }
            }
            return normalSet(NovaUtils.WHAT_SET_TIMING_VULUME_PARAM, NovaUtils.WHAT_SET_TIMING_VULUME_PARAM, bytes);
        }
    }

    public List<VolumeByTimeParam> getTimingVolumeParam() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_TIMING_VOLUME_PARAM, bytes,
                        bytes.length);
                String temp = AesUtil.toHex(outbuf, 0, outbuf.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf,
                        NovaUtils.WHAT_GET_TIMING_VOLUME_PARAM);

                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length >= 2) {
                    int length = recvBuf[1];
                    int itemSize = 15;
                    List<VolumeByTimeParam> list = new ArrayList<VolumeByTimeParam>();
                    for (int i = 0; i < length; i++) {
                        VolumeByTimeParam param = new VolumeByTimeParam();
                        int day = recvBuf[2 + i * itemSize];
                        int month = recvBuf[3 + i * itemSize] - 1;
                        int year = NovaUtils.bytesToShort(recvBuf, 4 + i * itemSize) - 1900;
                        int second = recvBuf[6 + i * itemSize];
                        int minute = recvBuf[7 + i * itemSize];
                        int hour = recvBuf[8 + i * itemSize];
                        param.setStart(new Date(year, month, day, hour, minute, second));

                        day = recvBuf[9 + i * itemSize];
                        month = recvBuf[10 + i * itemSize] - 1;
                        year = NovaUtils.bytesToShort(recvBuf, 11 + i * itemSize) - 1900;
                        second = recvBuf[13 + i * itemSize];
                        minute = recvBuf[14 + i * itemSize];
                        hour = recvBuf[15 + i * itemSize];
                        param.setEnd(new Date(year, month, day, hour, minute, second));

                        param.setVolume(recvBuf[16 + i * itemSize]);
                        list.add(param);
                    }
                    return list;
                }
                return null;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 点检需要配合lct点检功能使用，不能单独使用
     *
     * @return 大于0 为点检坏块总数。
     */
    public int spotCheckBadBlockInit() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SCREEN_POINT_DETECT_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_SCREEN_POINT_DETECT_RSP);
                if (recvBuf == null) {
                    return NovaUtils.SEND_NOT_RECV_ERROR;
                } else if (recvBuf.length != 4 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_RECV_ERROR;
                }

                return (0xffff & NovaUtils.bytesToShort(recvBuf, 2));
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 获取坏块信息
     *
     * @return BadBlockInfo
     */
    public String getSpotCheckBadBlockInfo() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SCREEN_POINT_DETECT_REQ, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[BIG_SIZE];
                // 消息回传判断
                byte recvBuf[] = recv(socket, inputBuf, NovaUtils.WHAT_SCREEN_POINT_DETECT_RSP);
                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length != 4 && recvBuf[1] != SUCCESS) {
                    return null;
                }

                outbuf = makeSendPacket(NovaUtils.WHAT_GET_ERROR_POINT_REQ1, bytes, bytes.length);
                socket.write(outbuf);

                int recvBufSize = 0;
                // 消息回传判断
                recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_ERROR_POINT_RSP1);
                if (recvBuf == null) {
                    return null;
                } else if (recvBuf.length != 4 && recvBuf[1] != SUCCESS) {
                    return null;
                }
                recvBufSize = 0xffff & NovaUtils.bytesToShort(recvBuf, 2);
                StringBuilder out = new StringBuilder();
                // 消息回传判断
                while (true) {
                    recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_ERROR_POINT_RSP2);
                    if (recvBuf == null) {
                        return null;
                    }
                    out.append(new String(recvBuf, 3, recvBuf.length - 3));
                    if (recvBuf.length < recvBufSize) {
                        return out.toString();
                    }
                }
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return null;
            } catch (IOException e) {
                e.printStackTrace();
                return null;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 开关继电器状态
     *
     * @param power1
     * @param power2
     * @param power3
     * @param power4
     * @return 状态码
     */
    public int setRelayPowers(boolean power1, boolean power2, boolean power3, boolean power4) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                byte[] bytes = new byte[4];
                bytes[0] = power1 ? (byte) 1 : 0;
                bytes[1] = power2 ? (byte) 1 : 0;
                bytes[2] = power3 ? (byte) 1 : 0;
                bytes[3] = power4 ? (byte) 1 : 0;
                byte[] outbuf = makeSendPacket(NovaUtils.SET_RELAY, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.SET_RELAY);
                if (recvBuf == null) {
                    return NovaUtils.SEND_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_RECV_ERROR;
                }

                return SUCCESS;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置路网图
     *
     * @param colors 全段的颜色 0 表示黑，1 表示红，2 表示绿，3 表示黄
     * @return 状态码
     */
    public int setRDSRoad(String colors) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                byte[] tempbuf = new byte[colors.length()];
                for (int i = 0; i < colors.length(); i++) {
                    tempbuf[i] = (byte) (colors.getBytes()[i] - 0x30);
                }
                byte[] outbuf = makeSendPacket(NovaUtils.SET_RDS_ROAD, tempbuf, tempbuf.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.SET_RDS_ROAD);
                if (recvBuf == null) {
                    return NovaUtils.SEND_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_RECV_ERROR;
                }

                return SUCCESS;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 设置路网图
     *
     * @return colors 全段的颜色
     */
    public String getRDSRoad() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[1];
                bytes[0] = 1;
                byte[] outbuf = makeSendPacket(NovaUtils.GET_RDS_ROAD, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte recvBuf[] = recv(socket, inputBuf, NovaUtils.GET_RDS_ROAD);
                if (recvBuf != null && recvBuf.length >= 1) {
                    for (int i = 0; i < recvBuf.length; i++) {
                        recvBuf[i] += 0x30;
                    }
                    return new String(recvBuf, 1, recvBuf.length - 1);
                }
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }

            return null;
        }
    }

    /**
     * 获取卡上全部播放列表
     *
     * @return
     */
    public List<Integer> getAllPlaylistId() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_ALL_PLAYLIST_ID, null, 0);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_ALL_PLAYLIST_ID);
                if (recvBuf != null && recvBuf.length >= 2) {

                    List<Integer> list = new ArrayList<>();
                    for (int i = 1; i < recvBuf.length; i++) {
                        list.add(recvBuf[i] & 0xff);
                    }
                    Collections.sort(list);
                    return list;

                }
            } catch (UnknownHostException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }

            return null;
        }
    }

    /**
     * 设置自动亮度参数
     *
     * @param second
     * @param count
     * @return
     */
    public int setAutoBrightnessTimeParam(byte second, byte count) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }

                byte[] tempbuf = new byte[2];
                tempbuf[0] = second;
                tempbuf[1] = count;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SET_AUTO_BRIGHTNESS_TIME_PARAM, tempbuf, tempbuf.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_SET_AUTO_BRIGHTNESS_TIME_PARAM);
                if (recvBuf == null) {
                    return NovaUtils.SEND_NOT_RECV_ERROR;
                } else if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_RECV_ERROR;
                }

                return SUCCESS;
            } catch (UnknownHostException e) {
                e.printStackTrace();
                return NovaUtils.UNKNOWN_HOST;
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }

    /**
     * 当一条发送，需要获取指定消息。
     *
     * @param iSocket
     * @param inputBuf
     * @return
     */
    private byte[] recv(ISocket iSocket, byte[] inputBuf, short what) {
        int failCount = 0;
        if (mProtocalParser == null) {
            if (mEncryptKey != null) {
                mProtocalParser = new ProtocalParser(encryptedType, mEncryptKey);
            } else {
                mProtocalParser = new ProtocalParser();
            }
        }

        return getRevBytes(iSocket, inputBuf, what, failCount);

    }

    private byte[] getRevBytes(ISocket iSocket, byte[] inputBuf, short what, int failCount) {
        while (true) {
            try {
                byte[] recvBuf;
                if (mProtocalParser.remainBuffer != null && !mProtocalParser.isNeedReload) {
                    recvBuf = mProtocalParser.recvProc(inputBuf, 0, what);
                } else {
                    int len = iSocket.read(inputBuf, 0, inputBuf.length);
                    recvBuf = mProtocalParser.recvProc(inputBuf, len, what);
                }
                if (recvBuf != null) {
                    return recvBuf;
                }
            } catch (IOException e) {
                return null;
            }

            failCount++;
            try {
                Thread.sleep(500);
            } catch (Exception e1) {
                LogUtil.log(e1.toString());
                Thread.currentThread().interrupt();
            }
            if (failCount >= MAX_FAIL_COUNT) {
                return null;
            }
        }
    }

    /**
     * 通用设置
     *
     * @param sendWhat
     * @param recvWhat
     * @param bytes
     * @return
     */
    private int normalSet(short sendWhat, short recvWhat, byte[] bytes) {
        ISocket socket = null;
        try {
            socket = getSocket();
            if (socket == null) {
                return NovaUtils.SOCKET_ERROR;
            }

            byte[] outbuf = makeSendPacket(sendWhat, bytes, bytes.length);
            String temp = AesUtil.toHex(outbuf, 0, outbuf.length);
            socket.write(outbuf);

            byte[] inputBuf = new byte[TEMP_SIZE];
            // 消息回传判断.
            byte[] recvBuf = recv(socket, inputBuf, recvWhat);
            if (recvBuf != null) {
                if (recvBuf.length >= 2 && recvBuf[1] != SUCCESS) {
                    return NovaUtils.SEND_RECV_ERROR;
                }
            } else {
                return NovaUtils.SEND_NOT_RECV_ERROR;
            }

            return SUCCESS;
        } catch (UnknownHostException e) {
            return NovaUtils.UNKNOWN_HOST;
        } catch (IOException e) {
            return NovaUtils.IO_ERROR;
        } finally {
            closeSocket(socket);
        }

    }

    /**
     * 获取Mac地址
     *
     * @return
     */
    public byte[] getMac() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_MAC, null, 0);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_MAC);
                if (recvBuf != null && recvBuf.length == 7) {
                    byte[] buf = new byte[6];
                    System.arraycopy(recvBuf, 1, buf, 0, 6);
                    return buf;
                }
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }
        }
        return null;
    }

    /**
     * 获取设备SN号
     *
     * @return
     */
    public String getSN() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_SN, null, 0);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_SN);
                if (recvBuf != null && recvBuf.length > 1) {
                    byte[] buf = new byte[recvBuf.length - 1];
                    System.arraycopy(recvBuf, 1, buf, 0, buf.length);
                    return new String(buf);
                }

            } catch (UnknownHostException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }
        }
        return null;
    }

    /**
     * 获取媒体文件
     *
     * @return
     */
    public boolean getDeviceScreenshot2() {
        byte[] bytes = new byte[1];
        int i = normalSet(NovaUtils.WHAT_GET_SCREENSHOT2, NovaUtils.WHAT_GET_SCREENSHOT2, bytes);
        return i == SUCCESS;

    }

    /**
     * 获取媒体文件
     *
     * @param savePath
     * @param fileName
     * @return
     */
    public boolean getMediaFile(String savePath, String fileName, long timeOut) {
        FileReceiveTask fileReceiveTask = new FileReceiveTask(savePath, fileName);
        boolean b = fileReceiveTask.prepare();
        if (!b) {
            return false;
        }
        FileServerBean bean = new FileServerBean();
        bean.setIp(FileServer.getInstance().getIp());
        bean.setPort(FileServer.getInstance().getPort());
        bean.setUuidStr(fileReceiveTask.getId());
        bean.setFileName(fileName);
        byte[] bytes = null;
        try {
            bytes = getBytes(new Gson().toJson(bean));
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }
        int i = normalSet(NovaUtils.WHAT_GET_MEDIA_FILE, NovaUtils.WHAT_GET_MEDIA_FILE, bytes);
        if (i == SUCCESS) {
            b = fileReceiveTask.receive(timeOut);
            fileReceiveTask.release();
            return b;
        } else {
            fileReceiveTask.release();
            return false;
        }
    }

    /**
     * 根据id清理播放文件
     *
     * @param id
     * @return
     */
    public int deletePlaylistById(int id) {
        byte[] bytes = new byte[1];
        bytes[0] = (byte) id;
        return normalSet(NovaUtils.WHAT_DELETE_PLAYLIST_BY_ID, NovaUtils.WHAT_DELETE_PLAYLIST_BY_ID, bytes);
    }

    /**
     * 删除媒体文件
     *
     * @param fileName
     * @return
     */
    public int deleteMeidaFile(String fileName) {
        byte[] bytes = null;
        try {
            bytes = getBytes(fileName);
        } catch (Exception e) {
            LogUtil.log("deleteMeidaFile:" + e.toString());
        }
        if (bytes != null) {
            return normalSet(NovaUtils.WHAT_DELETE_MEDIA_FILE, NovaUtils.WHAT_DELETE_MEDIA_FILE, bytes);
        }
        return -1;
    }

    /**
     * 获取所有媒体文件名称
     *
     * @return
     */
    @Override
    public List<String> getAllMediaFileName() {
        ISocket socket = null;
        try {
            synchronized (this) {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_ALL_MEDIA_FILENAME, null, 0);
                socket.write(outbuf);
                byte[] inputBuf = new byte[TEMP_SIZE];
                // 消息回传判断.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_ALL_MEDIA_FILENAME);
                assert recvBuf != null;
                if (recvBuf.length > 1) {
                    byte[] buf = new byte[recvBuf.length - 1];
                    System.arraycopy(recvBuf, 1, buf, 0, buf.length);
                    String str = new String(buf, StandardCharsets.UTF_8);
                    if (!str.isEmpty()) {
                        return new Gson().fromJson(str, new TypeToken<List<String>>() {
                        }.getType());

                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            closeSocket(socket);
        }
        return null;

    }

    /**
     * 设置rc4继电器
     *
     * @param b0
     * @param b1
     * @param b2
     * @param b3
     * @return
     */
    public int setRelay(boolean b0, boolean b1, boolean b2, boolean b3) {
        byte[] bytes = new byte[4];
        bytes[0] = (byte) (b0 ? 0x01 : 0x00);
        bytes[1] = (byte) (b1 ? 0x01 : 0x00);
        bytes[2] = (byte) (b2 ? 0x01 : 0x00);
        bytes[3] = (byte) (b3 ? 0x01 : 0x00);
        return this.normalSet(NovaUtils.SET_RELAY, NovaUtils.SET_RELAY, bytes);
    }

    /**
     * 设置播放配置
     *
     * @param isAntialias   是否抗锯齿
     * @param isTxtAdaptive 是否自适应
     * @return
     */
    public int setPlayConfig(boolean isAntialias, boolean isTxtAdaptive) {
        byte[] bytes = new byte[2];
        bytes[0] = (byte) (isAntialias ? 0x01 : 0x00);
        bytes[1] = (byte) (isTxtAdaptive ? 0x01 : 0x00);
        return this.normalSet(NovaUtils.WHAT_SET_PLAY_CONFIG, NovaUtils.WHAT_SET_PLAY_CONFIG, bytes);
    }

    /**
     * 获取播放设置
     *
     * @return
     */
    public byte[] getPlayConfig() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_PLAY_CONFIG, null, 0);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_PLAY_CONFIG);
                if (recvBuf != null && recvBuf.length > 1) {
                    byte[] buf = new byte[recvBuf.length - 1];
                    System.arraycopy(recvBuf, 1, buf, 0, buf.length);
                    return buf;
                }

            } catch (UnknownHostException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }
        }
        return null;
    }

    /**
     * 设置重启策略
     *
     * @param policy 重启策略
     * @return
     */
    public int setRebootPolicy(TimingPolicy policy) throws UnsupportedEncodingException {
        byte[] bytes = getBytes(new Gson().toJson(policy));
        return this.normalSet(NovaUtils.WHAT_SET_TIME_POLICY, NovaUtils.WHAT_SET_TIME_POLICY, bytes);
    }

    /**
     * 移除重启策略
     *
     * @return
     */
    public int removeRebootPolicy() {
        byte[] bytes = new byte[1];
        bytes[0] = TimingPolicy.TYPE_REBOOT;
        return this.normalSet(NovaUtils.WHAT_REMOVE_TIME_POLICY, NovaUtils.WHAT_REMOVE_TIME_POLICY, bytes);
    }

    /**
     * 获取重启策略
     *
     * @return
     */
    public TimingPolicy getRebootPolicy() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[1];
                bytes[0] = TimingPolicy.TYPE_REBOOT;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_TIME_POLICY, bytes, 1);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_TIME_POLICY);
                if (recvBuf != null && recvBuf.length >= 3) {
                    String result = new SocketResult(recvBuf).getResult();
                    return new Gson().fromJson(result, TimingPolicy.class);
                }
            } catch (UnknownHostException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }

            return null;
        }
    }

    /**
     * 获取内源支持分辨率
     *
     * @return
     */
    public String getSupportedResolution() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_SUPPORT_VIDEO_SOURCE, null, 0);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_SUPPORT_VIDEO_SOURCE);
                if (recvBuf != null && recvBuf.length > 1) {
                    byte[] buf = new byte[recvBuf.length - 1];
                    System.arraycopy(recvBuf, 1, buf, 0, buf.length);
                    return new String(buf);
                }

            } catch (UnknownHostException e) {
                e.printStackTrace();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }
        }
        return null;
    }

    /**
     * 设置自定义ping地址
     *
     * @param ip
     * @return
     */
    public int setPingAddress(String ip) {
        synchronized (this) {
            byte[] ipBytes;
            try {
                ipBytes = getBytes(ip);
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
                return NovaUtils.PARAM_ERROR;
            }
            return normalSet(NovaUtils.WHAT_SET_PING_CONFIG_IP, NovaUtils.WHAT_SET_PING_CONFIG_IP, ipBytes);
        }
    }

    /**
     * 获取自定义ping地址
     *
     * @return
     */
    public String getPingAddress() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[1];
                bytes[0] = 1;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_PING_CONFIG_IP, bytes, 1);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_PING_CONFIG_IP);
                return new SocketResult(recvBuf).getResult();
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }

            return null;
        }
    }

    /**
     * 获取屏幕亮度信息
     *
     * @return
     */
    public BrightnessBean getBrightnessParam() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[1];
                bytes[0] = 1;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_BRIGHTNESS_PARAM, bytes, 1);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_GET_BRIGHTNESS_PARAM);
                if (recvBuf != null && recvBuf.length >= 3) {
                    int rev = (recvBuf[2] & 0xff);
                    return new BrightnessBean(recvBuf[1], "" + (rev / 255.0));
                }
            } catch (IOException e) {
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }

            return null;
        }
    }

    /**
     * 路网图内容更新
     *
     * @param routes
     */
    public void updateTJRoad(List<TJRoute> routes) {
        try {
            String json = new Gson().toJson(routes);
            File file = File.createTempFile("item_road_0", ".load");
            byte[] bytes = json.getBytes(StandardCharsets.UTF_8);
            FileUtils.writeFileAppend(file, bytes, 0, bytes.length);
            int i = sendFile(file, "item_road_0.load");
            LogUtil.log("updateTJRoad 路网图更新状态：" + i);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * @return
     */
    public CpuAndMemoryInfo getCpuAndMemoryInfo() {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[0];
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_CPU_AND_MEMORY_RATE, bytes, 0);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte recvBuf[] = recv(socket, inputBuf, NovaUtils.WHAT_GET_CPU_AND_MEMORY_RATE);
                if (recvBuf != null && recvBuf.length >= 11) {
                    int cpu = (recvBuf[2] & 0xff);
                    int totalMemory = NovaUtils.bytesToInt(recvBuf, 3);
                    int freeMemory = NovaUtils.bytesToInt(recvBuf, 7);
                    CpuAndMemoryInfo info = new CpuAndMemoryInfo(SUCCESS, cpu, totalMemory, freeMemory);
                    return info;
                }
            } catch (Exception e) {

            }
            return null;
        }
    }

    public List<String> getFileListName(int type) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[1];
                bytes[0] = (byte) type;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_FILENAME_BY_TYPE, bytes, 1);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte recvBuf[] = recv(socket, inputBuf, NovaUtils.WHAT_GET_FILENAME_BY_TYPE);
                if (recvBuf != null && recvBuf.length > 1) {
                    byte[] buf = new byte[recvBuf.length - 1];
                    System.arraycopy(recvBuf, 1, buf, 0, buf.length);
                    String str = new String(buf, StandardCharsets.UTF_8);
                    if (str != null && !str.isEmpty()) {
                        List<String> list = new Gson().fromJson(str, new TypeToken<List<String>>() {
                        }.getType());
                        return list;
                    }
                }
            } catch (Exception e) {

            }
            return null;
        }
    }

    public int setAdvanceTiming(String policy) throws UnsupportedEncodingException {
        synchronized (this) {
            try {
                byte[] bytes = getBytes(policy);
                return this.normalSet(NovaUtils.WHAT_SET_TIME_POLICY, NovaUtils.WHAT_SET_TIME_POLICY, bytes);
            } catch (Exception e) {
                return -1;
            }
        }
    }

    public TimingPolicy getAdvanceTiming() throws UnsupportedEncodingException {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return null;
                }
                byte[] bytes = new byte[1];
                bytes[0] = TimingPolicy.TYPE_REBOOT;
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_GET_TIME_POLICY, bytes, 1);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte recvBuf[] = recv(mSocket, inputBuf, NovaUtils.WHAT_GET_TIME_POLICY);
                if (recvBuf != null && recvBuf.length >= 3) {
                    String result = new SocketResult(recvBuf).getResult();
                    TimingPolicy bean = new Gson().fromJson(result, TimingPolicy.class);
                    return bean;
                }
            } catch (UnknownHostException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            } finally {
                closeSocket(socket);
            }
            return null;
        }
    }

    public int removeAdvanceTiming() {
        byte[] bytes = new byte[1];
        bytes[0] = TimingPolicy.TYPE_REBOOT;
        return this.normalSet(NovaUtils.WHAT_REMOVE_TIME_POLICY, NovaUtils.WHAT_REMOVE_TIME_POLICY, bytes);
    }

    public int solidify(byte flag) {
        synchronized (this) {
            ISocket socket = null;
            try {
                socket = getSocket();
                if (socket == null) {
                    return NovaUtils.SOCKET_ERROR;
                }
                byte[] bytes = new byte[]{flag};
                byte[] outbuf = makeSendPacket(NovaUtils.WHAT_SOLIDIFY, bytes, bytes.length);
                socket.write(outbuf);

                byte[] inputBuf = new byte[TEMP_SIZE];
                // 发送文件名回传.
                byte[] recvBuf = recv(socket, inputBuf, NovaUtils.WHAT_SOLIDIFY);
                return new SocketResult(recvBuf).getCode();
            } catch (IOException e) {
                e.printStackTrace();
                return NovaUtils.IO_ERROR;
            } finally {
                closeSocket(socket);
            }
        }
    }
}
